/*
*
* Copyright (c) 2019 Texas Instruments Incorporated
*
* All rights reserved not granted herein.
*
* Limited License.
*
* Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive
* license under copyrights and patents it now or hereafter owns or controls to make,
* have made, use, import, offer to sell and sell ("Utilize") this software subject to the
* terms herein.  With respect to the foregoing patent license, such license is granted
* solely to the extent that any such patent is necessary to Utilize the software alone.
* The patent license shall not apply to any combinations which include this software,
* other than combinations with devices manufactured by or for TI ("TI Devices").
* No hardware patent is licensed hereunder.
*
* Redistributions must preserve existing copyright notices and reproduce this license
* (including the above copyright notice and the disclaimer and (if applicable) source
* code license limitations below) in the documentation and/or other materials provided
* with the distribution
*
* Redistribution and use in binary form, without modification, are permitted provided
* that the following conditions are met:
*
* *       No reverse engineering, decompilation, or disassembly of this software is
* permitted with respect to any software provided in binary form.
*
* *       any redistribution and use are licensed by TI for use only with TI Devices.
*
* *       Nothing shall obligate TI to provide you with source code for the software
* licensed and provided to you in object code.
*
* If software source code is provided to you, modification and redistribution of the
* source code are permitted provided that the following conditions are met:
*
* *       any redistribution and use of the source code, including any resulting derivative
* works, are licensed by TI for use only with TI Devices.
*
* *       any redistribution and use of any object code compiled from the source code
* and any resulting derivative works, are licensed by TI for use only with TI Devices.
*
* Neither the name of Texas Instruments Incorporated nor the names of its suppliers
*
* may be used to endorse or promote products derived from this software without
* specific prior written permission.
*
* DISCLAIMER.
*
* THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

/**
 *  \file     Cpsw_Stats.c
 *
 *  \brief    This file contains statistics related functionality.
 */

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include <string.h>
#include <ti/csl/csl_cpswitch.h>
#include <ti/drv/cpsw/soc/cpsw_soc.h>
#include <cpsw/Cpsw_Types.h>
#include <cpsw/Cpsw_Stats.h>

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                 Internal Function Declarations                             */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                            Global Variables                                */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                  Internal Function Definitions                             */
/* ========================================================================== */

static const CpswSoc_CpswIpInfo *CpswStats_getIpInfo(Cpsw_Type cpswType)
{
    uint32_t i;
    const CpswSoc_CpswIpInfo *ipInfo = NULL;
    const CpswSoc_CpswIpInfoTbl *ipInfoTbl = CpswSoc_getIPInfoTbl();
    bool foundIp = false;

    for (i = 0; i < ipInfoTbl->numCpswIPs; i++)
    {
        if (ipInfoTbl->info[i].cpswType == cpswType)
        {
            foundIp = true;
            break;
        }
    }

    if (foundIp)
    {
        ipInfo = &ipInfoTbl->info[i];
    }

    return ipInfo;
}


/* ========================================================================== */
/*                          Function Definitions                              */
/* ========================================================================== */

int32_t CpswStats_open(CpswStats *stats, Cpsw_Type cpswType)
{
    CSL_CPSW_PORTSTAT statEnable;
    const CpswSoc_CpswIpInfo *ipInfo = CpswStats_getIpInfo(cpswType);

    stats->regs = (CSL_Xge_cpswRegs *)ipInfo->cpswInfo.regBaseAddr;

    /* Clear all statistics counters */
    CSL_CPSW_getStats(stats->regs, (CSL_CPSW_STATS*)stats->stats);
    memset(stats->stats, 0U, CPSW_PORT_MAX * sizeof(CpswStats_Stats));

    /* Enable statistics in all ports */
    memset(&statEnable, 0U, sizeof(CSL_CPSW_PORTSTAT));
    statEnable.p0StatEnable = 1U;
    statEnable.p1StatEnable = 1U;
    CSL_CPSW_setPortStatsEnableReg(stats->regs, &statEnable);

    stats->isInit = TRUE;

    return CPSW_SOK;
}

void CpswStats_close(CpswStats *stats)
{
    CSL_CPSW_PORTSTAT statEnable;

    if (TRUE == stats->isInit)
    {
        /* Disable statistics in all ports */
        memset(&statEnable, 0U, sizeof(CSL_CPSW_PORTSTAT));
        CSL_CPSW_setPortStatsEnableReg(stats->regs, &statEnable);

        /* Clear all statistics counters */
        memset(stats->stats, 0U, CPSW_PORT_MAX * sizeof(CpswStats_Stats));

        stats->regs = NULL;
        stats->isInit = FALSE;
    }
}

void CpswStats_getStats(CpswStats *stats, CpswPort_Num num, CpswStats_Stats *val)
{
    CSL_CPSW_STATS st;
    uint32_t *readStats = (uint32_t *)&st;
    uint32_t *accStats;
    uint32_t block;
    uint32_t i;

    if (CPSW_PORT_MAX > num)
    {
        block = (uint32_t)num;
        accStats = (uint32_t *)&stats->stats[block];

        CSL_CPSW_getPortStats(stats->regs, block, &st);

        for (i = 0U; i < CPSW_STATS_BLOCK_ELEM_NUM; i++)
        {
            accStats[i] += readStats[i];
        }

        memcpy(val, &stats->stats[block], sizeof(CpswStats_Stats));
    }
}
