/*
*
* Copyright (c) 2019 Texas Instruments Incorporated
*
* All rights reserved not granted herein.
*
* Limited License.
*
* Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive
* license under copyrights and patents it now or hereafter owns or controls to make,
* have made, use, import, offer to sell and sell ("Utilize") this software subject to the
* terms herein.  With respect to the foregoing patent license, such license is granted
* solely to the extent that any such patent is necessary to Utilize the software alone.
* The patent license shall not apply to any combinations which include this software,
* other than combinations with devices manufactured by or for TI ("TI Devices").
* No hardware patent is licensed hereunder.
*
* Redistributions must preserve existing copyright notices and reproduce this license
* (including the above copyright notice and the disclaimer and (if applicable) source
* code license limitations below) in the documentation and/or other materials provided
* with the distribution
*
* Redistribution and use in binary form, without modification, are permitted provided
* that the following conditions are met:
*
* *       No reverse engineering, decompilation, or disassembly of this software is
* permitted with respect to any software provided in binary form.
*
* *       any redistribution and use are licensed by TI for use only with TI Devices.
*
* *       Nothing shall obligate TI to provide you with source code for the software
* licensed and provided to you in object code.
*
* If software source code is provided to you, modification and redistribution of the
* source code are permitted provided that the following conditions are met:
*
* *       any redistribution and use of the source code, including any resulting derivative
* works, are licensed by TI for use only with TI Devices.
*
* *       any redistribution and use of any object code compiled from the source code
* and any resulting derivative works, are licensed by TI for use only with TI Devices.
*
* Neither the name of Texas Instruments Incorporated nor the names of its suppliers
*
* may be used to endorse or promote products derived from this software without
* specific prior written permission.
*
* DISCLAIMER.
*
* THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

/**
 *  \file     Pwm_Priv.h
 *
 *  \brief    This file contains the common declarations and macros used across
 *            all the PWM MCAL driver files.
 *
 */

#ifndef PWM_PRIV_H_
#define PWM_PRIV_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include "Pwm.h"


/* IP Specific include */
#include <ti/csl/csl_timer.h>
#include <ti/csl/soc.h>
#if (STD_ON == PWM_DEV_ERROR_DETECT)
#include "Det.h"
#endif
#ifdef __cplusplus
extern "C"
{
#endif
/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */

#define PWMTIMER_MAX_RESOLUTION    (0xFFFFFFFFU)
#define PWMTICKMAXVALUE            (0xF0000000U)

/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */

/** \brief Pwm configuration structure internal to driver */
typedef struct
{
    boolean               channelForcedIdle;
    /**< PWM channel forced idle status array */
    uint32                baseAddr;
    /**< Base address of the PWM Channel */
    Pwm_ChannelConfigType chCfg;
    /**< Channel configurations */
    Pwm_ChannelConfigType_PC chCfg_PC;
    /**< ChannelId configuration in PC */
    uint32               channelNotifyActiveFlag;
    /**<  Variable indicate the notification status for each channel */
    Pwm_EdgeNotificationType Notification;
    /**<  Variable indicate the type of notification  each channel */
    uint8 channelState;
    /**< Variable to indicate the state of the channel */
} Pwm_ChObjType;

/* ========================================================================== */
/*                         GLOBAL VARIABLES                                   */
/* ========================================================================== */

extern volatile uint8 Pwm_DrvStatus;
extern volatile Pwm_OutputStateType Pwm_Intrflag[PWM_MAX_NUM_CHANNELS];

/* ========================================================================== */
/*                          Function Declarations                             */
/* ========================================================================== */
#if (PWM_SET_PERIOD_AND_DUTY_API==STD_ON) || \
    (PWM_SET_DUTY_CYCLE_API == STD_ON)
void Pwm_SetDutyCycle_Internal(Pwm_ChObjType *chObj, uint16 DutyCycle);
#endif
#if (STD_ON == PWM_DEV_ERROR_DETECT)
void Pwm_reportDetError(uint8 apiId, uint8 errorId);
#endif
void Pwm_ResetChObj(Pwm_ChObjType *chObj);
void Pwm_CopyConfig(Pwm_ChObjType *chObj, const Pwm_ChannelConfigType * chCfg,
                    const Pwm_ChannelConfigType_PC * chCfg_pc);
void Pwm_HwUnitInit(Pwm_ChObjType *chObj);
void Pwm_IpInit(const Pwm_ChObjType *chObj, uint32 baseAddr);
void Pwm_timerCounterInit(const Pwm_ChObjType *chObj, uint32 baseAddr);
#if (STD_ON == PWM_DEINIT_API)
void Pwm_IpDeInit( const Pwm_ChObjType *chObj);
#endif
void Pwm_IpCfgDutyCycle(
   uint32 baseAddr, uint32 dutycycle, uint32 hwPeriod);
uint32 Pwm_GetBaseAddr(Pwm_ChannelType ChannelNumber);
void Pwm_SetPolarity(uint16 dutyCycle, uint32 baseAddr,
                    Pwm_OutputStateType polarity);
void Pwm_IpCfgSetPolarity(uint16 dutyCycle, uint32 baseAddr,
                    Pwm_OutputStateType polarity);
void Pwm_SetIdleState(const Pwm_ChObjType *chObj);
void Pwm_IpCfgSetIdleState(const Pwm_ChObjType *chObj);
#if (STD_ON == PWM_REGISTER_READBACK_API)
void Pwm_IpRegisterReadback(uint32 baseAddr, 
           Pwm_RegisterReadbackType *RegRbPtr);
#endif
void Pwm_IpEnable(uint32 instanceClkHz, uint32 baseAddr);
void Pwm_IntDisableAll(Pwm_ChannelType pwmChannel);
void Pwm_IntrClear(Pwm_ChannelType pwmChannel);
#if (STD_ON == PWM_NOTIFICATION_SUPPORTED)
void Pwm_ChannelNotificationISR(Pwm_ChannelType ChannelNumber);
#endif  
#ifdef __cplusplus
}
#endif

#endif  /* #ifndef PWM_PRIV_H_ */
