/*
*
* Copyright (c) 2019 Texas Instruments Incorporated
*
* All rights reserved not granted herein.
*
* Limited License.
*
* Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive
* license under copyrights and patents it now or hereafter owns or controls to make,
* have made, use, import, offer to sell and sell ("Utilize") this software subject to the
* terms herein.  With respect to the foregoing patent license, such license is granted
* solely to the extent that any such patent is necessary to Utilize the software alone.
* The patent license shall not apply to any combinations which include this software,
* other than combinations with devices manufactured by or for TI ("TI Devices").
* No hardware patent is licensed hereunder.
*
* Redistributions must preserve existing copyright notices and reproduce this license
* (including the above copyright notice and the disclaimer and (if applicable) source
* code license limitations below) in the documentation and/or other materials provided
* with the distribution
*
* Redistribution and use in binary form, without modification, are permitted provided
* that the following conditions are met:
*
* *       No reverse engineering, decompilation, or disassembly of this software is
* permitted with respect to any software provided in binary form.
*
* *       any redistribution and use are licensed by TI for use only with TI Devices.
*
* *       Nothing shall obligate TI to provide you with source code for the software
* licensed and provided to you in object code.
*
* If software source code is provided to you, modification and redistribution of the
* source code are permitted provided that the following conditions are met:
*
* *       any redistribution and use of the source code, including any resulting derivative
* works, are licensed by TI for use only with TI Devices.
*
* *       any redistribution and use of any object code compiled from the source code
* and any resulting derivative works, are licensed by TI for use only with TI Devices.
*
* Neither the name of Texas Instruments Incorporated nor the names of its suppliers
*
* may be used to endorse or promote products derived from this software without
* specific prior written permission.
*
* DISCLAIMER.
*
* THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/
/***********************************************************************
    Project: Dio
    Date   : 2020-01-22 20:44:29

    This file is generated by EB Tresos
    Do not modify this file, otherwise the software may behave in
    unexpected way.

 ***********************************************************************/
/**
 *  \file     Dio_Cfg.h
 *
 *  \brief    This file contains generated pre compile configuration file
 *            for DIO MCAL driver
 *
 */

/**
 *  \defgroup MCAL_DIO_CFG DIO Configuration
 *
 *  This files defines DIO MCAL configuration structures
 *  @{
 */

#ifndef DIO_CFG_H_
#define DIO_CFG_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include "Dem.h"

#ifdef __cplusplus
extern "C" {
#endif


/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */
/** \brief DIO Driver Instance ID */
#define DIO_INSTANCE_ID                 ((uint8) 0U)

/**
 *  \name DIO service enable/diable defines
 *
 *  Pre-compile switches for enabling/disabling DIO MCAL APIs
 *  @{
 */
/** \brief Enable/Disable DEV error detection */
#define DIO_DEV_ERROR_DETECT            (STD_ON)

/** \brief Enable/Disable Dio_GetVersionInfo() */
#define DIO_VERSION_INFO_API            (STD_ON)

/** \brief Enable/Disable Dio_FlipChannel() */
#define DIO_FLIP_CHANNEL_API            (STD_ON)

/** \brief Enable/disable Dio register read back API */
#define DIO_REGISTER_READBACK_API       (STD_ON)
/* @} */

 /** \brief Max number of DIO ports */
#define DIO_ENABLED_PORT_MASK          (0U    | (1U << 0)\
 | (1U << 1)\
 | (1U << 6)\
 | (1U << 7)\
 | (1U << 8)\
 | (1U << 10)\
 | (1U << 11)\
)

/* Requirements : MCAL-1101, MCAL-1102, MCAL-1133 */
/**
 *  \name Symbolic name of DIO PORTs
 *
 *  This is symbolic name of DIO ports generated from configurator
 *  These defines map to physical GPIO register numbers.
 *  @{
 */
/** \brief Symbolic name for GPIO REG #0 */
#define DioConf_DioPort_WKUP_GPIO0_B01 ((Dio_PortType) 0U)
/** \brief Symbolic name for GPIO REG #1 */
#define DioConf_DioPort_WKUP_GPIO0_B23 ((Dio_PortType) 1U)
/** \brief Symbolic name for GPIO REG #6 */
#define DioConf_DioPort_GPIO0_B01 ((Dio_PortType) 6U)
/** \brief Symbolic name for GPIO REG #7 */
#define DioConf_DioPort_GPIO0_B23 ((Dio_PortType) 7U)
/** \brief Symbolic name for GPIO REG #8 */
#define DioConf_DioPort_GPIO0_B45 ((Dio_PortType) 8U)
/** \brief Symbolic name for GPIO REG #10 */
#define DioConf_DioPort_GPIO1_B01 ((Dio_PortType) 10U)
/** \brief Symbolic name for GPIO REG #11 */
#define DioConf_DioPort_GPIO1_B23 ((Dio_PortType) 11U)
/* @} */


/* Requirements :  MCAL-1101, MCAL-1102, MCAL-1129*/
/**
 *  \name Symbolic name of DIO Channels
 *
 *  This is symbolic name of DIO channels generated from configurator
 *  These defines map to GPIO register channels
 *  The channel numbers are mapped as
 *  WKUP_GPIO0_B01: Channel 0  - Channel 31
 *  WKUP_GPIO0_B23: Channel 32 - Channel 63
 *  WKUP_GPIO0_B45: Channel 64 - Channel 83
 *  WKUP_GPIO1_B01: Channel 144 - Channel 175
 *  WKUP_GPIO1_B23: Channel 176 - Channel 207
 *  WKUP_GPIO1_B45: Channel 208 - Channel 227
 *  GPIO0_B01: Channel 288 - Channel 319
 *  GPIO0_B23: Channel 320 - Channel 351
 *  GPIO0_B45: Channel 352  - Channel 383
 *  GPIO0_B67: Channel 384 - Channel 415
 *  GPIO1_B01: Channel 432 - Channel 463
 *  GPIO1_B23: Channel 464 - Channel 467
 *  GPIO2_B01: Channel 576 - Channel 607
 *  GPIO2_B23: Channel 608 - Channel 639
 *  GPIO2_B45: Channel 640 - Channel 671
 *  GPIO2_B67: Channel 672 - Channel 703
 *  GPIO3_B01: Channel 720 - Channel 751
 *  GPIO3_B23: Channel 752 - Channel 755
 *  GPIO4_B01: Channel 864 - Channel 895
 *  GPIO4_B23: Channel 896 - Channel 927
 *  GPIO4_B45: Channel 928 - Channel 959
 *  GPIO4_B67: Channel 960 - Channel 991
 *  GPIO5_B01: Channel 1008 - Channel 1039
 *  GPIO5_B23: Channel 1040 - Channel 1043
 *  GPIO6_B01: Channel 1152 - Channel 1183
 *  GPIO6_B23: Channel 1184 - Channel 1215
 *  GPIO6_B45: Channel 1216 - Channel 1247
 *  GPIO6_B67: Channel 1248 - Channel 1279
 *  GPIO7_B01: Channel 1296 - Channel 1327
 *  GPIO7_B23: Channel 1328 - Channel 1331
 *  @{
 */




/** \brief Symbolic name for GPIO channel #0
  *		   Can Transceiver Enable MCU MCAN0 */

#define DioConf_DioChannel_CAN_TRCV_MCU_MCAN_0 ((Dio_ChannelType) 0U)
/** \brief Shorter Symbolic name for GPIO channel #0 CAN_TRCV_MCU_MCAN_0 */
#define CAN_TRCV_MCU_MCAN_0 DioConf_DioChannel_CAN_TRCV_MCU_MCAN_0

/** \brief Symbolic name for GPIO channel #1 WKUP_GPIO0_B01_Ch1 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch1 ((Dio_ChannelType) 1U)
/** \brief Shorter Symbolic name for GPIO channel #1 WKUP_GPIO0_B01_Ch1 */
#define WKUP_GPIO0_B01_Ch1 DioConf_DioChannel_WKUP_GPIO0_B01_Ch1

/** \brief Symbolic name for GPIO channel #2
  *		   Can Transceiver Enable MCU MCAN1 */

#define DioConf_DioChannel_CAN_TRCV_MCU_MCAN_1 ((Dio_ChannelType) 2U)
/** \brief Shorter Symbolic name for GPIO channel #2 CAN_TRCV_MCU_MCAN_1 */
#define CAN_TRCV_MCU_MCAN_1 DioConf_DioChannel_CAN_TRCV_MCU_MCAN_1

/** \brief Symbolic name for GPIO channel #3 WKUP_GPIO0_B01_Ch3 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch3 ((Dio_ChannelType) 3U)
/** \brief Shorter Symbolic name for GPIO channel #3 WKUP_GPIO0_B01_Ch3 */
#define WKUP_GPIO0_B01_Ch3 DioConf_DioChannel_WKUP_GPIO0_B01_Ch3

/** \brief Symbolic name for GPIO channel #4 WKUP_GPIO0_B01_Ch4 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch4 ((Dio_ChannelType) 4U)
/** \brief Shorter Symbolic name for GPIO channel #4 WKUP_GPIO0_B01_Ch4 */
#define WKUP_GPIO0_B01_Ch4 DioConf_DioChannel_WKUP_GPIO0_B01_Ch4

/** \brief Symbolic name for GPIO channel #5 WKUP_GPIO0_B01_Ch5 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch5 ((Dio_ChannelType) 5U)
/** \brief Shorter Symbolic name for GPIO channel #5 WKUP_GPIO0_B01_Ch5 */
#define WKUP_GPIO0_B01_Ch5 DioConf_DioChannel_WKUP_GPIO0_B01_Ch5

/** \brief Symbolic name for GPIO channel #6 WKUP_GPIO0_B01_Ch6 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch6 ((Dio_ChannelType) 6U)
/** \brief Shorter Symbolic name for GPIO channel #6 WKUP_GPIO0_B01_Ch6 */
#define WKUP_GPIO0_B01_Ch6 DioConf_DioChannel_WKUP_GPIO0_B01_Ch6

/** \brief Symbolic name for GPIO channel #7 WKUP_GPIO0_B01_Ch7 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch7 ((Dio_ChannelType) 7U)
/** \brief Shorter Symbolic name for GPIO channel #7 WKUP_GPIO0_B01_Ch7 */
#define WKUP_GPIO0_B01_Ch7 DioConf_DioChannel_WKUP_GPIO0_B01_Ch7

/** \brief Symbolic name for GPIO channel #8 WKUP_GPIO0_B01_Ch8 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch8 ((Dio_ChannelType) 8U)
/** \brief Shorter Symbolic name for GPIO channel #8 WKUP_GPIO0_B01_Ch8 */
#define WKUP_GPIO0_B01_Ch8 DioConf_DioChannel_WKUP_GPIO0_B01_Ch8

/** \brief Symbolic name for GPIO channel #9 WKUP_GPIO0_B01_Ch9 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch9 ((Dio_ChannelType) 9U)
/** \brief Shorter Symbolic name for GPIO channel #9 WKUP_GPIO0_B01_Ch9 */
#define WKUP_GPIO0_B01_Ch9 DioConf_DioChannel_WKUP_GPIO0_B01_Ch9

/** \brief Symbolic name for GPIO channel #10 WKUP_GPIO0_B01_Ch10 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch10 ((Dio_ChannelType) 10U)
/** \brief Shorter Symbolic name for GPIO channel #10 WKUP_GPIO0_B01_Ch10 */
#define WKUP_GPIO0_B01_Ch10 DioConf_DioChannel_WKUP_GPIO0_B01_Ch10

/** \brief Symbolic name for GPIO channel #11 WKUP_GPIO0_B01_Ch11 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch11 ((Dio_ChannelType) 11U)
/** \brief Shorter Symbolic name for GPIO channel #11 WKUP_GPIO0_B01_Ch11 */
#define WKUP_GPIO0_B01_Ch11 DioConf_DioChannel_WKUP_GPIO0_B01_Ch11

/** \brief Symbolic name for GPIO channel #12 WKUP_GPIO0_B01_Ch12 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch12 ((Dio_ChannelType) 12U)
/** \brief Shorter Symbolic name for GPIO channel #12 WKUP_GPIO0_B01_Ch12 */
#define WKUP_GPIO0_B01_Ch12 DioConf_DioChannel_WKUP_GPIO0_B01_Ch12

/** \brief Symbolic name for GPIO channel #13 WKUP_GPIO0_B01_Ch13 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch13 ((Dio_ChannelType) 13U)
/** \brief Shorter Symbolic name for GPIO channel #13 WKUP_GPIO0_B01_Ch13 */
#define WKUP_GPIO0_B01_Ch13 DioConf_DioChannel_WKUP_GPIO0_B01_Ch13

/** \brief Symbolic name for GPIO channel #14 WKUP_GPIO0_B01_Ch14 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch14 ((Dio_ChannelType) 14U)
/** \brief Shorter Symbolic name for GPIO channel #14 WKUP_GPIO0_B01_Ch14 */
#define WKUP_GPIO0_B01_Ch14 DioConf_DioChannel_WKUP_GPIO0_B01_Ch14

/** \brief Symbolic name for GPIO channel #15 WKUP_GPIO0_B01_Ch15 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch15 ((Dio_ChannelType) 15U)
/** \brief Shorter Symbolic name for GPIO channel #15 WKUP_GPIO0_B01_Ch15 */
#define WKUP_GPIO0_B01_Ch15 DioConf_DioChannel_WKUP_GPIO0_B01_Ch15

/** \brief Symbolic name for GPIO channel #16 WKUP_GPIO0_B01_Ch16 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch16 ((Dio_ChannelType) 16U)
/** \brief Shorter Symbolic name for GPIO channel #16 WKUP_GPIO0_B01_Ch16 */
#define WKUP_GPIO0_B01_Ch16 DioConf_DioChannel_WKUP_GPIO0_B01_Ch16

/** \brief Symbolic name for GPIO channel #17 WKUP_GPIO0_B01_Ch17 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch17 ((Dio_ChannelType) 17U)
/** \brief Shorter Symbolic name for GPIO channel #17 WKUP_GPIO0_B01_Ch17 */
#define WKUP_GPIO0_B01_Ch17 DioConf_DioChannel_WKUP_GPIO0_B01_Ch17

/** \brief Symbolic name for GPIO channel #18 WKUP_GPIO0_B01_Ch18 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch18 ((Dio_ChannelType) 18U)
/** \brief Shorter Symbolic name for GPIO channel #18 WKUP_GPIO0_B01_Ch18 */
#define WKUP_GPIO0_B01_Ch18 DioConf_DioChannel_WKUP_GPIO0_B01_Ch18

/** \brief Symbolic name for GPIO channel #19 WKUP_GPIO0_B01_Ch19 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch19 ((Dio_ChannelType) 19U)
/** \brief Shorter Symbolic name for GPIO channel #19 WKUP_GPIO0_B01_Ch19 */
#define WKUP_GPIO0_B01_Ch19 DioConf_DioChannel_WKUP_GPIO0_B01_Ch19

/** \brief Symbolic name for GPIO channel #20 WKUP_GPIO0_B01_Ch20 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch20 ((Dio_ChannelType) 20U)
/** \brief Shorter Symbolic name for GPIO channel #20 WKUP_GPIO0_B01_Ch20 */
#define WKUP_GPIO0_B01_Ch20 DioConf_DioChannel_WKUP_GPIO0_B01_Ch20

/** \brief Symbolic name for GPIO channel #21 WKUP_GPIO0_B01_Ch21 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch21 ((Dio_ChannelType) 21U)
/** \brief Shorter Symbolic name for GPIO channel #21 WKUP_GPIO0_B01_Ch21 */
#define WKUP_GPIO0_B01_Ch21 DioConf_DioChannel_WKUP_GPIO0_B01_Ch21

/** \brief Symbolic name for GPIO channel #22 WKUP_GPIO0_B01_Ch22 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch22 ((Dio_ChannelType) 22U)
/** \brief Shorter Symbolic name for GPIO channel #22 WKUP_GPIO0_B01_Ch22 */
#define WKUP_GPIO0_B01_Ch22 DioConf_DioChannel_WKUP_GPIO0_B01_Ch22

/** \brief Symbolic name for GPIO channel #23 WKUP_GPIO0_B01_Ch23 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch23 ((Dio_ChannelType) 23U)
/** \brief Shorter Symbolic name for GPIO channel #23 WKUP_GPIO0_B01_Ch23 */
#define WKUP_GPIO0_B01_Ch23 DioConf_DioChannel_WKUP_GPIO0_B01_Ch23

/** \brief Symbolic name for GPIO channel #24 WKUP_GPIO0_B01_Ch24 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch24 ((Dio_ChannelType) 24U)
/** \brief Shorter Symbolic name for GPIO channel #24 WKUP_GPIO0_B01_Ch24 */
#define WKUP_GPIO0_B01_Ch24 DioConf_DioChannel_WKUP_GPIO0_B01_Ch24

/** \brief Symbolic name for GPIO channel #25 WKUP_GPIO0_B01_Ch25 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch25 ((Dio_ChannelType) 25U)
/** \brief Shorter Symbolic name for GPIO channel #25 WKUP_GPIO0_B01_Ch25 */
#define WKUP_GPIO0_B01_Ch25 DioConf_DioChannel_WKUP_GPIO0_B01_Ch25

/** \brief Symbolic name for GPIO channel #26 WKUP_GPIO0_B01_Ch26 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch26 ((Dio_ChannelType) 26U)
/** \brief Shorter Symbolic name for GPIO channel #26 WKUP_GPIO0_B01_Ch26 */
#define WKUP_GPIO0_B01_Ch26 DioConf_DioChannel_WKUP_GPIO0_B01_Ch26

/** \brief Symbolic name for GPIO channel #27 WKUP_GPIO0_B01_Ch27 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch27 ((Dio_ChannelType) 27U)
/** \brief Shorter Symbolic name for GPIO channel #27 WKUP_GPIO0_B01_Ch27 */
#define WKUP_GPIO0_B01_Ch27 DioConf_DioChannel_WKUP_GPIO0_B01_Ch27

/** \brief Symbolic name for GPIO channel #28 WKUP_GPIO0_B01_Ch28 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch28 ((Dio_ChannelType) 28U)
/** \brief Shorter Symbolic name for GPIO channel #28 WKUP_GPIO0_B01_Ch28 */
#define WKUP_GPIO0_B01_Ch28 DioConf_DioChannel_WKUP_GPIO0_B01_Ch28

/** \brief Symbolic name for GPIO channel #29 WKUP_GPIO0_B01_Ch29 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch29 ((Dio_ChannelType) 29U)
/** \brief Shorter Symbolic name for GPIO channel #29 WKUP_GPIO0_B01_Ch29 */
#define WKUP_GPIO0_B01_Ch29 DioConf_DioChannel_WKUP_GPIO0_B01_Ch29

/** \brief Symbolic name for GPIO channel #30 WKUP_GPIO0_B01_Ch30 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch30 ((Dio_ChannelType) 30U)
/** \brief Shorter Symbolic name for GPIO channel #30 WKUP_GPIO0_B01_Ch30 */
#define WKUP_GPIO0_B01_Ch30 DioConf_DioChannel_WKUP_GPIO0_B01_Ch30

/** \brief Symbolic name for GPIO channel #31 WKUP_GPIO0_B01_Ch31 */

#define DioConf_DioChannel_WKUP_GPIO0_B01_Ch31 ((Dio_ChannelType) 31U)
/** \brief Shorter Symbolic name for GPIO channel #31 WKUP_GPIO0_B01_Ch31 */
#define WKUP_GPIO0_B01_Ch31 DioConf_DioChannel_WKUP_GPIO0_B01_Ch31

/** \brief Symbolic name for GPIO channel #32 WKUP_GPIO0_B23_Ch0 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch0 ((Dio_ChannelType) 32U)
/** \brief Shorter Symbolic name for GPIO channel #32 WKUP_GPIO0_B23_Ch0 */
#define WKUP_GPIO0_B23_Ch0 DioConf_DioChannel_WKUP_GPIO0_B23_Ch0

/** \brief Symbolic name for GPIO channel #33 WKUP_GPIO0_B23_Ch1 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch1 ((Dio_ChannelType) 33U)
/** \brief Shorter Symbolic name for GPIO channel #33 WKUP_GPIO0_B23_Ch1 */
#define WKUP_GPIO0_B23_Ch1 DioConf_DioChannel_WKUP_GPIO0_B23_Ch1

/** \brief Symbolic name for GPIO channel #34 WKUP_GPIO0_B23_Ch2 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch2 ((Dio_ChannelType) 34U)
/** \brief Shorter Symbolic name for GPIO channel #34 WKUP_GPIO0_B23_Ch2 */
#define WKUP_GPIO0_B23_Ch2 DioConf_DioChannel_WKUP_GPIO0_B23_Ch2

/** \brief Symbolic name for GPIO channel #35 WKUP_GPIO0_B23_Ch3 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch3 ((Dio_ChannelType) 35U)
/** \brief Shorter Symbolic name for GPIO channel #35 WKUP_GPIO0_B23_Ch3 */
#define WKUP_GPIO0_B23_Ch3 DioConf_DioChannel_WKUP_GPIO0_B23_Ch3

/** \brief Symbolic name for GPIO channel #36 WKUP_GPIO0_B23_Ch4 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch4 ((Dio_ChannelType) 36U)
/** \brief Shorter Symbolic name for GPIO channel #36 WKUP_GPIO0_B23_Ch4 */
#define WKUP_GPIO0_B23_Ch4 DioConf_DioChannel_WKUP_GPIO0_B23_Ch4

/** \brief Symbolic name for GPIO channel #37 WKUP_GPIO0_B23_Ch5 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch5 ((Dio_ChannelType) 37U)
/** \brief Shorter Symbolic name for GPIO channel #37 WKUP_GPIO0_B23_Ch5 */
#define WKUP_GPIO0_B23_Ch5 DioConf_DioChannel_WKUP_GPIO0_B23_Ch5

/** \brief Symbolic name for GPIO channel #38 WKUP_GPIO0_B23_Ch6 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch6 ((Dio_ChannelType) 38U)
/** \brief Shorter Symbolic name for GPIO channel #38 WKUP_GPIO0_B23_Ch6 */
#define WKUP_GPIO0_B23_Ch6 DioConf_DioChannel_WKUP_GPIO0_B23_Ch6

/** \brief Symbolic name for GPIO channel #39 WKUP_GPIO0_B23_Ch7 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch7 ((Dio_ChannelType) 39U)
/** \brief Shorter Symbolic name for GPIO channel #39 WKUP_GPIO0_B23_Ch7 */
#define WKUP_GPIO0_B23_Ch7 DioConf_DioChannel_WKUP_GPIO0_B23_Ch7

/** \brief Symbolic name for GPIO channel #40 WKUP_GPIO0_B23_Ch8 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch8 ((Dio_ChannelType) 40U)
/** \brief Shorter Symbolic name for GPIO channel #40 WKUP_GPIO0_B23_Ch8 */
#define WKUP_GPIO0_B23_Ch8 DioConf_DioChannel_WKUP_GPIO0_B23_Ch8

/** \brief Symbolic name for GPIO channel #41 WKUP_GPIO0_B23_Ch9 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch9 ((Dio_ChannelType) 41U)
/** \brief Shorter Symbolic name for GPIO channel #41 WKUP_GPIO0_B23_Ch9 */
#define WKUP_GPIO0_B23_Ch9 DioConf_DioChannel_WKUP_GPIO0_B23_Ch9

/** \brief Symbolic name for GPIO channel #42 WKUP_GPIO0_B23_Ch10 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch10 ((Dio_ChannelType) 42U)
/** \brief Shorter Symbolic name for GPIO channel #42 WKUP_GPIO0_B23_Ch10 */
#define WKUP_GPIO0_B23_Ch10 DioConf_DioChannel_WKUP_GPIO0_B23_Ch10

/** \brief Symbolic name for GPIO channel #43 WKUP_GPIO0_B23_Ch11 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch11 ((Dio_ChannelType) 43U)
/** \brief Shorter Symbolic name for GPIO channel #43 WKUP_GPIO0_B23_Ch11 */
#define WKUP_GPIO0_B23_Ch11 DioConf_DioChannel_WKUP_GPIO0_B23_Ch11

/** \brief Symbolic name for GPIO channel #44 WKUP_GPIO0_B23_Ch12 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch12 ((Dio_ChannelType) 44U)
/** \brief Shorter Symbolic name for GPIO channel #44 WKUP_GPIO0_B23_Ch12 */
#define WKUP_GPIO0_B23_Ch12 DioConf_DioChannel_WKUP_GPIO0_B23_Ch12

/** \brief Symbolic name for GPIO channel #45 WKUP_GPIO0_B23_Ch13 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch13 ((Dio_ChannelType) 45U)
/** \brief Shorter Symbolic name for GPIO channel #45 WKUP_GPIO0_B23_Ch13 */
#define WKUP_GPIO0_B23_Ch13 DioConf_DioChannel_WKUP_GPIO0_B23_Ch13

/** \brief Symbolic name for GPIO channel #46 WKUP_GPIO0_B23_Ch14 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch14 ((Dio_ChannelType) 46U)
/** \brief Shorter Symbolic name for GPIO channel #46 WKUP_GPIO0_B23_Ch14 */
#define WKUP_GPIO0_B23_Ch14 DioConf_DioChannel_WKUP_GPIO0_B23_Ch14

/** \brief Symbolic name for GPIO channel #47 WKUP_GPIO0_B23_Ch15 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch15 ((Dio_ChannelType) 47U)
/** \brief Shorter Symbolic name for GPIO channel #47 WKUP_GPIO0_B23_Ch15 */
#define WKUP_GPIO0_B23_Ch15 DioConf_DioChannel_WKUP_GPIO0_B23_Ch15

/** \brief Symbolic name for GPIO channel #48 WKUP_GPIO0_B23_Ch16 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch16 ((Dio_ChannelType) 48U)
/** \brief Shorter Symbolic name for GPIO channel #48 WKUP_GPIO0_B23_Ch16 */
#define WKUP_GPIO0_B23_Ch16 DioConf_DioChannel_WKUP_GPIO0_B23_Ch16

/** \brief Symbolic name for GPIO channel #49 WKUP_GPIO0_B23_Ch17 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch17 ((Dio_ChannelType) 49U)
/** \brief Shorter Symbolic name for GPIO channel #49 WKUP_GPIO0_B23_Ch17 */
#define WKUP_GPIO0_B23_Ch17 DioConf_DioChannel_WKUP_GPIO0_B23_Ch17

/** \brief Symbolic name for GPIO channel #50 WKUP_GPIO0_B23_Ch18 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch18 ((Dio_ChannelType) 50U)
/** \brief Shorter Symbolic name for GPIO channel #50 WKUP_GPIO0_B23_Ch18 */
#define WKUP_GPIO0_B23_Ch18 DioConf_DioChannel_WKUP_GPIO0_B23_Ch18

/** \brief Symbolic name for GPIO channel #51 WKUP_GPIO0_B23_Ch19 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch19 ((Dio_ChannelType) 51U)
/** \brief Shorter Symbolic name for GPIO channel #51 WKUP_GPIO0_B23_Ch19 */
#define WKUP_GPIO0_B23_Ch19 DioConf_DioChannel_WKUP_GPIO0_B23_Ch19

/** \brief Symbolic name for GPIO channel #52 WKUP_GPIO0_B23_Ch20 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch20 ((Dio_ChannelType) 52U)
/** \brief Shorter Symbolic name for GPIO channel #52 WKUP_GPIO0_B23_Ch20 */
#define WKUP_GPIO0_B23_Ch20 DioConf_DioChannel_WKUP_GPIO0_B23_Ch20

/** \brief Symbolic name for GPIO channel #53 WKUP_GPIO0_B23_Ch21 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch21 ((Dio_ChannelType) 53U)
/** \brief Shorter Symbolic name for GPIO channel #53 WKUP_GPIO0_B23_Ch21 */
#define WKUP_GPIO0_B23_Ch21 DioConf_DioChannel_WKUP_GPIO0_B23_Ch21

/** \brief Symbolic name for GPIO channel #54
  *		   Can Transceiver Enable MCU MCAN0 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch22 ((Dio_ChannelType) 54U)
/** \brief Shorter Symbolic name for GPIO channel #54 WKUP_GPIO0_B23_Ch22 */
#define WKUP_GPIO0_B23_Ch22 DioConf_DioChannel_WKUP_GPIO0_B23_Ch22

/** \brief Symbolic name for GPIO channel #55 WKUP_GPIO0_B23_Ch23 */

#define DioConf_DioChannel_WKUP_GPIO0_B23_Ch23 ((Dio_ChannelType) 55U)
/** \brief Shorter Symbolic name for GPIO channel #55 WKUP_GPIO0_B23_Ch23 */
#define WKUP_GPIO0_B23_Ch23 DioConf_DioChannel_WKUP_GPIO0_B23_Ch23

/** \brief Symbolic name for GPIO channel #288 GPIO0_B01_Ch0 */

#define DioConf_DioChannel_GPIO0_B01_Ch0 ((Dio_ChannelType) 288U)
/** \brief Shorter Symbolic name for GPIO channel #288 GPIO0_B01_Ch0 */
#define GPIO0_B01_Ch0 DioConf_DioChannel_GPIO0_B01_Ch0

/** \brief Symbolic name for GPIO channel #289 GPIO0_B01_Ch1 */

#define DioConf_DioChannel_GPIO0_B01_Ch1 ((Dio_ChannelType) 289U)
/** \brief Shorter Symbolic name for GPIO channel #289 GPIO0_B01_Ch1 */
#define GPIO0_B01_Ch1 DioConf_DioChannel_GPIO0_B01_Ch1

/** \brief Symbolic name for GPIO channel #290 GPIO0_B01_Ch2 */

#define DioConf_DioChannel_GPIO0_B01_Ch2 ((Dio_ChannelType) 290U)
/** \brief Shorter Symbolic name for GPIO channel #290 GPIO0_B01_Ch2 */
#define GPIO0_B01_Ch2 DioConf_DioChannel_GPIO0_B01_Ch2

/** \brief Symbolic name for GPIO channel #291 GPIO0_B01_Ch3 */

#define DioConf_DioChannel_GPIO0_B01_Ch3 ((Dio_ChannelType) 291U)
/** \brief Shorter Symbolic name for GPIO channel #291 GPIO0_B01_Ch3 */
#define GPIO0_B01_Ch3 DioConf_DioChannel_GPIO0_B01_Ch3

/** \brief Symbolic name for GPIO channel #292 GPIO0_B01_Ch4 */

#define DioConf_DioChannel_GPIO0_B01_Ch4 ((Dio_ChannelType) 292U)
/** \brief Shorter Symbolic name for GPIO channel #292 GPIO0_B01_Ch4 */
#define GPIO0_B01_Ch4 DioConf_DioChannel_GPIO0_B01_Ch4

/** \brief Symbolic name for GPIO channel #293 GPIO0_B01_Ch5 */

#define DioConf_DioChannel_GPIO0_B01_Ch5 ((Dio_ChannelType) 293U)
/** \brief Shorter Symbolic name for GPIO channel #293 GPIO0_B01_Ch5 */
#define GPIO0_B01_Ch5 DioConf_DioChannel_GPIO0_B01_Ch5

/** \brief Symbolic name for GPIO channel #294 GPIO0_B01_Ch6 */

#define DioConf_DioChannel_GPIO0_B01_Ch6 ((Dio_ChannelType) 294U)
/** \brief Shorter Symbolic name for GPIO channel #294 GPIO0_B01_Ch6 */
#define GPIO0_B01_Ch6 DioConf_DioChannel_GPIO0_B01_Ch6

/** \brief Symbolic name for GPIO channel #295 GPIO0_B01_Ch7 */

#define DioConf_DioChannel_GPIO0_B01_Ch7 ((Dio_ChannelType) 295U)
/** \brief Shorter Symbolic name for GPIO channel #295 GPIO0_B01_Ch7 */
#define GPIO0_B01_Ch7 DioConf_DioChannel_GPIO0_B01_Ch7

/** \brief Symbolic name for GPIO channel #296 GPIO0_B01_Ch8 */

#define DioConf_DioChannel_GPIO0_B01_Ch8 ((Dio_ChannelType) 296U)
/** \brief Shorter Symbolic name for GPIO channel #296 GPIO0_B01_Ch8 */
#define GPIO0_B01_Ch8 DioConf_DioChannel_GPIO0_B01_Ch8

/** \brief Symbolic name for GPIO channel #297 GPIO0_B01_Ch9 */

#define DioConf_DioChannel_GPIO0_B01_Ch9 ((Dio_ChannelType) 297U)
/** \brief Shorter Symbolic name for GPIO channel #297 GPIO0_B01_Ch9 */
#define GPIO0_B01_Ch9 DioConf_DioChannel_GPIO0_B01_Ch9

/** \brief Symbolic name for GPIO channel #298 GPIO0_B01_Ch10 */

#define DioConf_DioChannel_GPIO0_B01_Ch10 ((Dio_ChannelType) 298U)
/** \brief Shorter Symbolic name for GPIO channel #298 GPIO0_B01_Ch10 */
#define GPIO0_B01_Ch10 DioConf_DioChannel_GPIO0_B01_Ch10

/** \brief Symbolic name for GPIO channel #299 GPIO0_B01_Ch11 */

#define DioConf_DioChannel_GPIO0_B01_Ch11 ((Dio_ChannelType) 299U)
/** \brief Shorter Symbolic name for GPIO channel #299 GPIO0_B01_Ch11 */
#define GPIO0_B01_Ch11 DioConf_DioChannel_GPIO0_B01_Ch11

/** \brief Symbolic name for GPIO channel #300 GPIO0_B01_Ch12 */

#define DioConf_DioChannel_GPIO0_B01_Ch12 ((Dio_ChannelType) 300U)
/** \brief Shorter Symbolic name for GPIO channel #300 GPIO0_B01_Ch12 */
#define GPIO0_B01_Ch12 DioConf_DioChannel_GPIO0_B01_Ch12

/** \brief Symbolic name for GPIO channel #301 GPIO0_B01_Ch13 */

#define DioConf_DioChannel_GPIO0_B01_Ch13 ((Dio_ChannelType) 301U)
/** \brief Shorter Symbolic name for GPIO channel #301 GPIO0_B01_Ch13 */
#define GPIO0_B01_Ch13 DioConf_DioChannel_GPIO0_B01_Ch13

/** \brief Symbolic name for GPIO channel #302 GPIO0_B01_Ch14 */

#define DioConf_DioChannel_GPIO0_B01_Ch14 ((Dio_ChannelType) 302U)
/** \brief Shorter Symbolic name for GPIO channel #302 GPIO0_B01_Ch14 */
#define GPIO0_B01_Ch14 DioConf_DioChannel_GPIO0_B01_Ch14

/** \brief Symbolic name for GPIO channel #303 GPIO0_B01_Ch15 */

#define DioConf_DioChannel_GPIO0_B01_Ch15 ((Dio_ChannelType) 303U)
/** \brief Shorter Symbolic name for GPIO channel #303 GPIO0_B01_Ch15 */
#define GPIO0_B01_Ch15 DioConf_DioChannel_GPIO0_B01_Ch15

/** \brief Symbolic name for GPIO channel #304 GPIO0_B01_Ch16 */

#define DioConf_DioChannel_GPIO0_B01_Ch16 ((Dio_ChannelType) 304U)
/** \brief Shorter Symbolic name for GPIO channel #304 GPIO0_B01_Ch16 */
#define GPIO0_B01_Ch16 DioConf_DioChannel_GPIO0_B01_Ch16

/** \brief Symbolic name for GPIO channel #305 GPIO0_B01_Ch17 */

#define DioConf_DioChannel_GPIO0_B01_Ch17 ((Dio_ChannelType) 305U)
/** \brief Shorter Symbolic name for GPIO channel #305 GPIO0_B01_Ch17 */
#define GPIO0_B01_Ch17 DioConf_DioChannel_GPIO0_B01_Ch17

/** \brief Symbolic name for GPIO channel #306 GPIO0_B01_Ch18 */

#define DioConf_DioChannel_GPIO0_B01_Ch18 ((Dio_ChannelType) 306U)
/** \brief Shorter Symbolic name for GPIO channel #306 GPIO0_B01_Ch18 */
#define GPIO0_B01_Ch18 DioConf_DioChannel_GPIO0_B01_Ch18

/** \brief Symbolic name for GPIO channel #307 GPIO0_B01_Ch19 */

#define DioConf_DioChannel_GPIO0_B01_Ch19 ((Dio_ChannelType) 307U)
/** \brief Shorter Symbolic name for GPIO channel #307 GPIO0_B01_Ch19 */
#define GPIO0_B01_Ch19 DioConf_DioChannel_GPIO0_B01_Ch19

/** \brief Symbolic name for GPIO channel #308 GPIO0_B01_Ch20 */

#define DioConf_DioChannel_GPIO0_B01_Ch20 ((Dio_ChannelType) 308U)
/** \brief Shorter Symbolic name for GPIO channel #308 GPIO0_B01_Ch20 */
#define GPIO0_B01_Ch20 DioConf_DioChannel_GPIO0_B01_Ch20

/** \brief Symbolic name for GPIO channel #309 GPIO0_B01_Ch21 */

#define DioConf_DioChannel_GPIO0_B01_Ch21 ((Dio_ChannelType) 309U)
/** \brief Shorter Symbolic name for GPIO channel #309 GPIO0_B01_Ch21 */
#define GPIO0_B01_Ch21 DioConf_DioChannel_GPIO0_B01_Ch21

/** \brief Symbolic name for GPIO channel #310 GPIO0_B01_Ch22 */

#define DioConf_DioChannel_GPIO0_B01_Ch22 ((Dio_ChannelType) 310U)
/** \brief Shorter Symbolic name for GPIO channel #310 GPIO0_B01_Ch22 */
#define GPIO0_B01_Ch22 DioConf_DioChannel_GPIO0_B01_Ch22

/** \brief Symbolic name for GPIO channel #311 GPIO0_B01_Ch23 */

#define DioConf_DioChannel_GPIO0_B01_Ch23 ((Dio_ChannelType) 311U)
/** \brief Shorter Symbolic name for GPIO channel #311 GPIO0_B01_Ch23 */
#define GPIO0_B01_Ch23 DioConf_DioChannel_GPIO0_B01_Ch23

/** \brief Symbolic name for GPIO channel #312 GPIO0_B01_Ch24 */

#define DioConf_DioChannel_GPIO0_B01_Ch24 ((Dio_ChannelType) 312U)
/** \brief Shorter Symbolic name for GPIO channel #312 GPIO0_B01_Ch24 */
#define GPIO0_B01_Ch24 DioConf_DioChannel_GPIO0_B01_Ch24

/** \brief Symbolic name for GPIO channel #313 GPIO0_B01_Ch25 */

#define DioConf_DioChannel_GPIO0_B01_Ch25 ((Dio_ChannelType) 313U)
/** \brief Shorter Symbolic name for GPIO channel #313 GPIO0_B01_Ch25 */
#define GPIO0_B01_Ch25 DioConf_DioChannel_GPIO0_B01_Ch25

/** \brief Symbolic name for GPIO channel #314 GPIO0_B01_Ch26 */

#define DioConf_DioChannel_GPIO0_B01_Ch26 ((Dio_ChannelType) 314U)
/** \brief Shorter Symbolic name for GPIO channel #314 GPIO0_B01_Ch26 */
#define GPIO0_B01_Ch26 DioConf_DioChannel_GPIO0_B01_Ch26

/** \brief Symbolic name for GPIO channel #315 GPIO0_B01_Ch27 */

#define DioConf_DioChannel_GPIO0_B01_Ch27 ((Dio_ChannelType) 315U)
/** \brief Shorter Symbolic name for GPIO channel #315 GPIO0_B01_Ch27 */
#define GPIO0_B01_Ch27 DioConf_DioChannel_GPIO0_B01_Ch27

/** \brief Symbolic name for GPIO channel #316 GPIO0_B01_Ch28 */

#define DioConf_DioChannel_GPIO0_B01_Ch28 ((Dio_ChannelType) 316U)
/** \brief Shorter Symbolic name for GPIO channel #316 GPIO0_B01_Ch28 */
#define GPIO0_B01_Ch28 DioConf_DioChannel_GPIO0_B01_Ch28

/** \brief Symbolic name for GPIO channel #317 GPIO0_B01_Ch29 */

#define DioConf_DioChannel_GPIO0_B01_Ch29 ((Dio_ChannelType) 317U)
/** \brief Shorter Symbolic name for GPIO channel #317 GPIO0_B01_Ch29 */
#define GPIO0_B01_Ch29 DioConf_DioChannel_GPIO0_B01_Ch29

/** \brief Symbolic name for GPIO channel #318 GPIO0_B01_Ch30 */

#define DioConf_DioChannel_GPIO0_B01_Ch30 ((Dio_ChannelType) 318U)
/** \brief Shorter Symbolic name for GPIO channel #318 GPIO0_B01_Ch30 */
#define GPIO0_B01_Ch30 DioConf_DioChannel_GPIO0_B01_Ch30

/** \brief Symbolic name for GPIO channel #319 GPIO0_B01_Ch31 */

#define DioConf_DioChannel_GPIO0_B01_Ch31 ((Dio_ChannelType) 319U)
/** \brief Shorter Symbolic name for GPIO channel #319 GPIO0_B01_Ch31 */
#define GPIO0_B01_Ch31 DioConf_DioChannel_GPIO0_B01_Ch31

/** \brief Symbolic name for GPIO channel #320 GPIO0_B23_Ch0 */

#define DioConf_DioChannel_GPIO0_B23_Ch0 ((Dio_ChannelType) 320U)
/** \brief Shorter Symbolic name for GPIO channel #320 GPIO0_B23_Ch0 */
#define GPIO0_B23_Ch0 DioConf_DioChannel_GPIO0_B23_Ch0

/** \brief Symbolic name for GPIO channel #321 GPIO0_B23_Ch1 */

#define DioConf_DioChannel_GPIO0_B23_Ch1 ((Dio_ChannelType) 321U)
/** \brief Shorter Symbolic name for GPIO channel #321 GPIO0_B23_Ch1 */
#define GPIO0_B23_Ch1 DioConf_DioChannel_GPIO0_B23_Ch1

/** \brief Symbolic name for GPIO channel #322 GPIO0_B23_Ch2 */

#define DioConf_DioChannel_GPIO0_B23_Ch2 ((Dio_ChannelType) 322U)
/** \brief Shorter Symbolic name for GPIO channel #322 GPIO0_B23_Ch2 */
#define GPIO0_B23_Ch2 DioConf_DioChannel_GPIO0_B23_Ch2

/** \brief Symbolic name for GPIO channel #323 GPIO0_B23_Ch3 */

#define DioConf_DioChannel_GPIO0_B23_Ch3 ((Dio_ChannelType) 323U)
/** \brief Shorter Symbolic name for GPIO channel #323 GPIO0_B23_Ch3 */
#define GPIO0_B23_Ch3 DioConf_DioChannel_GPIO0_B23_Ch3

/** \brief Symbolic name for GPIO channel #324 GPIO0_B23_Ch4 */

#define DioConf_DioChannel_GPIO0_B23_Ch4 ((Dio_ChannelType) 324U)
/** \brief Shorter Symbolic name for GPIO channel #324 GPIO0_B23_Ch4 */
#define GPIO0_B23_Ch4 DioConf_DioChannel_GPIO0_B23_Ch4

/** \brief Symbolic name for GPIO channel #325 GPIO0_B23_Ch5 */

#define DioConf_DioChannel_GPIO0_B23_Ch5 ((Dio_ChannelType) 325U)
/** \brief Shorter Symbolic name for GPIO channel #325 GPIO0_B23_Ch5 */
#define GPIO0_B23_Ch5 DioConf_DioChannel_GPIO0_B23_Ch5

/** \brief Symbolic name for GPIO channel #326 GPIO0_B23_Ch6 */

#define DioConf_DioChannel_GPIO0_B23_Ch6 ((Dio_ChannelType) 326U)
/** \brief Shorter Symbolic name for GPIO channel #326 GPIO0_B23_Ch6 */
#define GPIO0_B23_Ch6 DioConf_DioChannel_GPIO0_B23_Ch6

/** \brief Symbolic name for GPIO channel #327 GPIO0_B23_Ch7 */

#define DioConf_DioChannel_GPIO0_B23_Ch7 ((Dio_ChannelType) 327U)
/** \brief Shorter Symbolic name for GPIO channel #327 GPIO0_B23_Ch7 */
#define GPIO0_B23_Ch7 DioConf_DioChannel_GPIO0_B23_Ch7

/** \brief Symbolic name for GPIO channel #328 GPIO0_B23_Ch8 */

#define DioConf_DioChannel_GPIO0_B23_Ch8 ((Dio_ChannelType) 328U)
/** \brief Shorter Symbolic name for GPIO channel #328 GPIO0_B23_Ch8 */
#define GPIO0_B23_Ch8 DioConf_DioChannel_GPIO0_B23_Ch8

/** \brief Symbolic name for GPIO channel #329 GPIO0_B23_Ch9 */

#define DioConf_DioChannel_GPIO0_B23_Ch9 ((Dio_ChannelType) 329U)
/** \brief Shorter Symbolic name for GPIO channel #329 GPIO0_B23_Ch9 */
#define GPIO0_B23_Ch9 DioConf_DioChannel_GPIO0_B23_Ch9

/** \brief Symbolic name for GPIO channel #330 GPIO0_B23_Ch10 */

#define DioConf_DioChannel_GPIO0_B23_Ch10 ((Dio_ChannelType) 330U)
/** \brief Shorter Symbolic name for GPIO channel #330 GPIO0_B23_Ch10 */
#define GPIO0_B23_Ch10 DioConf_DioChannel_GPIO0_B23_Ch10

/** \brief Symbolic name for GPIO channel #331 GPIO0_B23_Ch11 */

#define DioConf_DioChannel_GPIO0_B23_Ch11 ((Dio_ChannelType) 331U)
/** \brief Shorter Symbolic name for GPIO channel #331 GPIO0_B23_Ch11 */
#define GPIO0_B23_Ch11 DioConf_DioChannel_GPIO0_B23_Ch11

/** \brief Symbolic name for GPIO channel #332 GPIO0_B23_Ch12 */

#define DioConf_DioChannel_GPIO0_B23_Ch12 ((Dio_ChannelType) 332U)
/** \brief Shorter Symbolic name for GPIO channel #332 GPIO0_B23_Ch12 */
#define GPIO0_B23_Ch12 DioConf_DioChannel_GPIO0_B23_Ch12

/** \brief Symbolic name for GPIO channel #333 GPIO0_B23_Ch13 */

#define DioConf_DioChannel_GPIO0_B23_Ch13 ((Dio_ChannelType) 333U)
/** \brief Shorter Symbolic name for GPIO channel #333 GPIO0_B23_Ch13 */
#define GPIO0_B23_Ch13 DioConf_DioChannel_GPIO0_B23_Ch13

/** \brief Symbolic name for GPIO channel #334 GPIO0_B23_Ch14 */

#define DioConf_DioChannel_GPIO0_B23_Ch14 ((Dio_ChannelType) 334U)
/** \brief Shorter Symbolic name for GPIO channel #334 GPIO0_B23_Ch14 */
#define GPIO0_B23_Ch14 DioConf_DioChannel_GPIO0_B23_Ch14

/** \brief Symbolic name for GPIO channel #335 GPIO0_B23_Ch15 */

#define DioConf_DioChannel_GPIO0_B23_Ch15 ((Dio_ChannelType) 335U)
/** \brief Shorter Symbolic name for GPIO channel #335 GPIO0_B23_Ch15 */
#define GPIO0_B23_Ch15 DioConf_DioChannel_GPIO0_B23_Ch15

/** \brief Symbolic name for GPIO channel #336 GPIO0_B23_Ch16 */

#define DioConf_DioChannel_GPIO0_B23_Ch16 ((Dio_ChannelType) 336U)
/** \brief Shorter Symbolic name for GPIO channel #336 GPIO0_B23_Ch16 */
#define GPIO0_B23_Ch16 DioConf_DioChannel_GPIO0_B23_Ch16

/** \brief Symbolic name for GPIO channel #337 GPIO0_B23_Ch17 */

#define DioConf_DioChannel_GPIO0_B23_Ch17 ((Dio_ChannelType) 337U)
/** \brief Shorter Symbolic name for GPIO channel #337 GPIO0_B23_Ch17 */
#define GPIO0_B23_Ch17 DioConf_DioChannel_GPIO0_B23_Ch17

/** \brief Symbolic name for GPIO channel #338 GPIO0_B23_Ch18 */

#define DioConf_DioChannel_GPIO0_B23_Ch18 ((Dio_ChannelType) 338U)
/** \brief Shorter Symbolic name for GPIO channel #338 GPIO0_B23_Ch18 */
#define GPIO0_B23_Ch18 DioConf_DioChannel_GPIO0_B23_Ch18

/** \brief Symbolic name for GPIO channel #339 GPIO0_B23_Ch19 */

#define DioConf_DioChannel_GPIO0_B23_Ch19 ((Dio_ChannelType) 339U)
/** \brief Shorter Symbolic name for GPIO channel #339 GPIO0_B23_Ch19 */
#define GPIO0_B23_Ch19 DioConf_DioChannel_GPIO0_B23_Ch19

/** \brief Symbolic name for GPIO channel #340 GPIO0_B23_Ch20 */

#define DioConf_DioChannel_GPIO0_B23_Ch20 ((Dio_ChannelType) 340U)
/** \brief Shorter Symbolic name for GPIO channel #340 GPIO0_B23_Ch20 */
#define GPIO0_B23_Ch20 DioConf_DioChannel_GPIO0_B23_Ch20

/** \brief Symbolic name for GPIO channel #341 GPIO0_B23_Ch21 */

#define DioConf_DioChannel_GPIO0_B23_Ch21 ((Dio_ChannelType) 341U)
/** \brief Shorter Symbolic name for GPIO channel #341 GPIO0_B23_Ch21 */
#define GPIO0_B23_Ch21 DioConf_DioChannel_GPIO0_B23_Ch21

/** \brief Symbolic name for GPIO channel #342 GPIO0_B23_Ch22 */

#define DioConf_DioChannel_GPIO0_B23_Ch22 ((Dio_ChannelType) 342U)
/** \brief Shorter Symbolic name for GPIO channel #342 GPIO0_B23_Ch22 */
#define GPIO0_B23_Ch22 DioConf_DioChannel_GPIO0_B23_Ch22

/** \brief Symbolic name for GPIO channel #343 GPIO0_B23_Ch23 */

#define DioConf_DioChannel_GPIO0_B23_Ch23 ((Dio_ChannelType) 343U)
/** \brief Shorter Symbolic name for GPIO channel #343 GPIO0_B23_Ch23 */
#define GPIO0_B23_Ch23 DioConf_DioChannel_GPIO0_B23_Ch23

/** \brief Symbolic name for GPIO channel #344 GPIO0_B23_Ch24 */

#define DioConf_DioChannel_GPIO0_B23_Ch24 ((Dio_ChannelType) 344U)
/** \brief Shorter Symbolic name for GPIO channel #344 GPIO0_B23_Ch24 */
#define GPIO0_B23_Ch24 DioConf_DioChannel_GPIO0_B23_Ch24

/** \brief Symbolic name for GPIO channel #345 GPIO0_B23_Ch25 */

#define DioConf_DioChannel_GPIO0_B23_Ch25 ((Dio_ChannelType) 345U)
/** \brief Shorter Symbolic name for GPIO channel #345 GPIO0_B23_Ch25 */
#define GPIO0_B23_Ch25 DioConf_DioChannel_GPIO0_B23_Ch25

/** \brief Symbolic name for GPIO channel #346 GPIO0_B23_Ch26 */

#define DioConf_DioChannel_GPIO0_B23_Ch26 ((Dio_ChannelType) 346U)
/** \brief Shorter Symbolic name for GPIO channel #346 GPIO0_B23_Ch26 */
#define GPIO0_B23_Ch26 DioConf_DioChannel_GPIO0_B23_Ch26

/** \brief Symbolic name for GPIO channel #347 GPIO0_B23_Ch27 */

#define DioConf_DioChannel_GPIO0_B23_Ch27 ((Dio_ChannelType) 347U)
/** \brief Shorter Symbolic name for GPIO channel #347 GPIO0_B23_Ch27 */
#define GPIO0_B23_Ch27 DioConf_DioChannel_GPIO0_B23_Ch27

/** \brief Symbolic name for GPIO channel #348
  *		   Can Transceiver Enable Main Domain MCAN instances 4,5,6,7,9,11 */

#define DioConf_DioChannel_CAN_TRCV_MAIN_DOMAIN_4_9_11 ((Dio_ChannelType) 348U)
/** \brief Shorter Symbolic name for GPIO channel #348 CAN_TRCV_MAIN_DOMAIN_4_9_11 */
#define CAN_TRCV_MAIN_DOMAIN_4_9_11 DioConf_DioChannel_CAN_TRCV_MAIN_DOMAIN_4_9_11

/** \brief Symbolic name for GPIO channel #349 GPIO0_B23_Ch29 */

#define DioConf_DioChannel_GPIO0_B23_Ch29 ((Dio_ChannelType) 349U)
/** \brief Shorter Symbolic name for GPIO channel #349 GPIO0_B23_Ch29 */
#define GPIO0_B23_Ch29 DioConf_DioChannel_GPIO0_B23_Ch29

/** \brief Symbolic name for GPIO channel #350 GPIO0_B23_Ch30 */

#define DioConf_DioChannel_GPIO0_B23_Ch30 ((Dio_ChannelType) 350U)
/** \brief Shorter Symbolic name for GPIO channel #350 GPIO0_B23_Ch30 */
#define GPIO0_B23_Ch30 DioConf_DioChannel_GPIO0_B23_Ch30

/** \brief Symbolic name for GPIO channel #351 GPIO0_B23_Ch31 */

#define DioConf_DioChannel_GPIO0_B23_Ch31 ((Dio_ChannelType) 351U)
/** \brief Shorter Symbolic name for GPIO channel #351 GPIO0_B23_Ch31 */
#define GPIO0_B23_Ch31 DioConf_DioChannel_GPIO0_B23_Ch31

/** \brief Symbolic name for GPIO channel #352 GPIO0_B45_Ch0 */

#define DioConf_DioChannel_GPIO0_B45_Ch0 ((Dio_ChannelType) 352U)
/** \brief Shorter Symbolic name for GPIO channel #352 GPIO0_B45_Ch0 */
#define GPIO0_B45_Ch0 DioConf_DioChannel_GPIO0_B45_Ch0

/** \brief Symbolic name for GPIO channel #353 GPIO0_B45_Ch1 */

#define DioConf_DioChannel_GPIO0_B45_Ch1 ((Dio_ChannelType) 353U)
/** \brief Shorter Symbolic name for GPIO channel #353 GPIO0_B45_Ch1 */
#define GPIO0_B45_Ch1 DioConf_DioChannel_GPIO0_B45_Ch1

/** \brief Symbolic name for GPIO channel #354 GPIO0_B45_Ch2 */

#define DioConf_DioChannel_GPIO0_B45_Ch2 ((Dio_ChannelType) 354U)
/** \brief Shorter Symbolic name for GPIO channel #354 GPIO0_B45_Ch2 */
#define GPIO0_B45_Ch2 DioConf_DioChannel_GPIO0_B45_Ch2

/** \brief Symbolic name for GPIO channel #355 GPIO0_B45_Ch3 */

#define DioConf_DioChannel_GPIO0_B45_Ch3 ((Dio_ChannelType) 355U)
/** \brief Shorter Symbolic name for GPIO channel #355 GPIO0_B45_Ch3 */
#define GPIO0_B45_Ch3 DioConf_DioChannel_GPIO0_B45_Ch3

/** \brief Symbolic name for GPIO channel #356 GPIO0_B45_Ch4 */

#define DioConf_DioChannel_GPIO0_B45_Ch4 ((Dio_ChannelType) 356U)
/** \brief Shorter Symbolic name for GPIO channel #356 GPIO0_B45_Ch4 */
#define GPIO0_B45_Ch4 DioConf_DioChannel_GPIO0_B45_Ch4

/** \brief Symbolic name for GPIO channel #357 GPIO0_B45_Ch5 */

#define DioConf_DioChannel_GPIO0_B45_Ch5 ((Dio_ChannelType) 357U)
/** \brief Shorter Symbolic name for GPIO channel #357 GPIO0_B45_Ch5 */
#define GPIO0_B45_Ch5 DioConf_DioChannel_GPIO0_B45_Ch5

/** \brief Symbolic name for GPIO channel #358 GPIO0_B45_Ch6 */

#define DioConf_DioChannel_GPIO0_B45_Ch6 ((Dio_ChannelType) 358U)
/** \brief Shorter Symbolic name for GPIO channel #358 GPIO0_B45_Ch6 */
#define GPIO0_B45_Ch6 DioConf_DioChannel_GPIO0_B45_Ch6

/** \brief Symbolic name for GPIO channel #359 GPIO0_B45_Ch7 */

#define DioConf_DioChannel_GPIO0_B45_Ch7 ((Dio_ChannelType) 359U)
/** \brief Shorter Symbolic name for GPIO channel #359 GPIO0_B45_Ch7 */
#define GPIO0_B45_Ch7 DioConf_DioChannel_GPIO0_B45_Ch7

/** \brief Symbolic name for GPIO channel #360 GPIO0_B45_Ch8 */

#define DioConf_DioChannel_GPIO0_B45_Ch8 ((Dio_ChannelType) 360U)
/** \brief Shorter Symbolic name for GPIO channel #360 GPIO0_B45_Ch8 */
#define GPIO0_B45_Ch8 DioConf_DioChannel_GPIO0_B45_Ch8

/** \brief Symbolic name for GPIO channel #361 GPIO0_B45_Ch9 */

#define DioConf_DioChannel_GPIO0_B45_Ch9 ((Dio_ChannelType) 361U)
/** \brief Shorter Symbolic name for GPIO channel #361 GPIO0_B45_Ch9 */
#define GPIO0_B45_Ch9 DioConf_DioChannel_GPIO0_B45_Ch9

/** \brief Symbolic name for GPIO channel #362 GPIO0_B45_Ch10 */

#define DioConf_DioChannel_GPIO0_B45_Ch10 ((Dio_ChannelType) 362U)
/** \brief Shorter Symbolic name for GPIO channel #362 GPIO0_B45_Ch10 */
#define GPIO0_B45_Ch10 DioConf_DioChannel_GPIO0_B45_Ch10

/** \brief Symbolic name for GPIO channel #363 GPIO0_B45_Ch11 */

#define DioConf_DioChannel_GPIO0_B45_Ch11 ((Dio_ChannelType) 363U)
/** \brief Shorter Symbolic name for GPIO channel #363 GPIO0_B45_Ch11 */
#define GPIO0_B45_Ch11 DioConf_DioChannel_GPIO0_B45_Ch11

/** \brief Symbolic name for GPIO channel #364 GPIO0_B45_Ch12 */

#define DioConf_DioChannel_GPIO0_B45_Ch12 ((Dio_ChannelType) 364U)
/** \brief Shorter Symbolic name for GPIO channel #364 GPIO0_B45_Ch12 */
#define GPIO0_B45_Ch12 DioConf_DioChannel_GPIO0_B45_Ch12

/** \brief Symbolic name for GPIO channel #365 GPIO0_B45_Ch13 */

#define DioConf_DioChannel_GPIO0_B45_Ch13 ((Dio_ChannelType) 365U)
/** \brief Shorter Symbolic name for GPIO channel #365 GPIO0_B45_Ch13 */
#define GPIO0_B45_Ch13 DioConf_DioChannel_GPIO0_B45_Ch13

/** \brief Symbolic name for GPIO channel #366 GPIO0_B45_Ch14 */

#define DioConf_DioChannel_GPIO0_B45_Ch14 ((Dio_ChannelType) 366U)
/** \brief Shorter Symbolic name for GPIO channel #366 GPIO0_B45_Ch14 */
#define GPIO0_B45_Ch14 DioConf_DioChannel_GPIO0_B45_Ch14

/** \brief Symbolic name for GPIO channel #367 GPIO0_B45_Ch15 */

#define DioConf_DioChannel_GPIO0_B45_Ch15 ((Dio_ChannelType) 367U)
/** \brief Shorter Symbolic name for GPIO channel #367 GPIO0_B45_Ch15 */
#define GPIO0_B45_Ch15 DioConf_DioChannel_GPIO0_B45_Ch15

/** \brief Symbolic name for GPIO channel #368 GPIO0_B45_Ch16 */

#define DioConf_DioChannel_GPIO0_B45_Ch16 ((Dio_ChannelType) 368U)
/** \brief Shorter Symbolic name for GPIO channel #368 GPIO0_B45_Ch16 */
#define GPIO0_B45_Ch16 DioConf_DioChannel_GPIO0_B45_Ch16

/** \brief Symbolic name for GPIO channel #369 GPIO0_B45_Ch17 */

#define DioConf_DioChannel_GPIO0_B45_Ch17 ((Dio_ChannelType) 369U)
/** \brief Shorter Symbolic name for GPIO channel #369 GPIO0_B45_Ch17 */
#define GPIO0_B45_Ch17 DioConf_DioChannel_GPIO0_B45_Ch17

/** \brief Symbolic name for GPIO channel #370 GPIO0_B45_Ch18 */

#define DioConf_DioChannel_GPIO0_B45_Ch18 ((Dio_ChannelType) 370U)
/** \brief Shorter Symbolic name for GPIO channel #370 GPIO0_B45_Ch18 */
#define GPIO0_B45_Ch18 DioConf_DioChannel_GPIO0_B45_Ch18

/** \brief Symbolic name for GPIO channel #371 GPIO0_B45_Ch19 */

#define DioConf_DioChannel_GPIO0_B45_Ch19 ((Dio_ChannelType) 371U)
/** \brief Shorter Symbolic name for GPIO channel #371 GPIO0_B45_Ch19 */
#define GPIO0_B45_Ch19 DioConf_DioChannel_GPIO0_B45_Ch19

/** \brief Symbolic name for GPIO channel #372 GPIO0_B45_Ch20 */

#define DioConf_DioChannel_GPIO0_B45_Ch20 ((Dio_ChannelType) 372U)
/** \brief Shorter Symbolic name for GPIO channel #372 GPIO0_B45_Ch20 */
#define GPIO0_B45_Ch20 DioConf_DioChannel_GPIO0_B45_Ch20

/** \brief Symbolic name for GPIO channel #373 GPIO0_B45_Ch21 */

#define DioConf_DioChannel_GPIO0_B45_Ch21 ((Dio_ChannelType) 373U)
/** \brief Shorter Symbolic name for GPIO channel #373 GPIO0_B45_Ch21 */
#define GPIO0_B45_Ch21 DioConf_DioChannel_GPIO0_B45_Ch21

/** \brief Symbolic name for GPIO channel #374 GPIO0_B45_Ch22 */

#define DioConf_DioChannel_GPIO0_B45_Ch22 ((Dio_ChannelType) 374U)
/** \brief Shorter Symbolic name for GPIO channel #374 GPIO0_B45_Ch22 */
#define GPIO0_B45_Ch22 DioConf_DioChannel_GPIO0_B45_Ch22

/** \brief Symbolic name for GPIO channel #375 GPIO0_B45_Ch23 */

#define DioConf_DioChannel_GPIO0_B45_Ch23 ((Dio_ChannelType) 375U)
/** \brief Shorter Symbolic name for GPIO channel #375 GPIO0_B45_Ch23 */
#define GPIO0_B45_Ch23 DioConf_DioChannel_GPIO0_B45_Ch23

/** \brief Symbolic name for GPIO channel #376 GPIO0_B45_Ch24 */

#define DioConf_DioChannel_GPIO0_B45_Ch24 ((Dio_ChannelType) 376U)
/** \brief Shorter Symbolic name for GPIO channel #376 GPIO0_B45_Ch24 */
#define GPIO0_B45_Ch24 DioConf_DioChannel_GPIO0_B45_Ch24

/** \brief Symbolic name for GPIO channel #377 GPIO0_B45_Ch25 */

#define DioConf_DioChannel_GPIO0_B45_Ch25 ((Dio_ChannelType) 377U)
/** \brief Shorter Symbolic name for GPIO channel #377 GPIO0_B45_Ch25 */
#define GPIO0_B45_Ch25 DioConf_DioChannel_GPIO0_B45_Ch25

/** \brief Symbolic name for GPIO channel #378 GPIO0_B45_Ch26 */

#define DioConf_DioChannel_GPIO0_B45_Ch26 ((Dio_ChannelType) 378U)
/** \brief Shorter Symbolic name for GPIO channel #378 GPIO0_B45_Ch26 */
#define GPIO0_B45_Ch26 DioConf_DioChannel_GPIO0_B45_Ch26

/** \brief Symbolic name for GPIO channel #379 GPIO0_B45_Ch27 */

#define DioConf_DioChannel_GPIO0_B45_Ch27 ((Dio_ChannelType) 379U)
/** \brief Shorter Symbolic name for GPIO channel #379 GPIO0_B45_Ch27 */
#define GPIO0_B45_Ch27 DioConf_DioChannel_GPIO0_B45_Ch27

/** \brief Symbolic name for GPIO channel #380 GPIO0_B45_Ch28 */

#define DioConf_DioChannel_GPIO0_B45_Ch28 ((Dio_ChannelType) 380U)
/** \brief Shorter Symbolic name for GPIO channel #380 GPIO0_B45_Ch28 */
#define GPIO0_B45_Ch28 DioConf_DioChannel_GPIO0_B45_Ch28

/** \brief Symbolic name for GPIO channel #381 GPIO0_B45_Ch29 */

#define DioConf_DioChannel_GPIO0_B45_Ch29 ((Dio_ChannelType) 381U)
/** \brief Shorter Symbolic name for GPIO channel #381 GPIO0_B45_Ch29 */
#define GPIO0_B45_Ch29 DioConf_DioChannel_GPIO0_B45_Ch29

/** \brief Symbolic name for GPIO channel #382 GPIO0_B45_Ch30 */

#define DioConf_DioChannel_GPIO0_B45_Ch30 ((Dio_ChannelType) 382U)
/** \brief Shorter Symbolic name for GPIO channel #382 GPIO0_B45_Ch30 */
#define GPIO0_B45_Ch30 DioConf_DioChannel_GPIO0_B45_Ch30

/** \brief Symbolic name for GPIO channel #383 GPIO0_B45_Ch31 */

#define DioConf_DioChannel_GPIO0_B45_Ch31 ((Dio_ChannelType) 383U)
/** \brief Shorter Symbolic name for GPIO channel #383 GPIO0_B45_Ch31 */
#define GPIO0_B45_Ch31 DioConf_DioChannel_GPIO0_B45_Ch31

/** \brief Symbolic name for GPIO channel #432 GPIO1_B01_Ch0 */

#define DioConf_DioChannel_GPIO1_B01_Ch0 ((Dio_ChannelType) 432U)
/** \brief Shorter Symbolic name for GPIO channel #432 GPIO1_B01_Ch0 */
#define GPIO1_B01_Ch0 DioConf_DioChannel_GPIO1_B01_Ch0

/** \brief Symbolic name for GPIO channel #433 GPIO1_B01_Ch1 */

#define DioConf_DioChannel_GPIO1_B01_Ch1 ((Dio_ChannelType) 433U)
/** \brief Shorter Symbolic name for GPIO channel #433 GPIO1_B01_Ch1 */
#define GPIO1_B01_Ch1 DioConf_DioChannel_GPIO1_B01_Ch1

/** \brief Symbolic name for GPIO channel #434 GPIO1_B01_Ch2 */

#define DioConf_DioChannel_GPIO1_B01_Ch2 ((Dio_ChannelType) 434U)
/** \brief Shorter Symbolic name for GPIO channel #434 GPIO1_B01_Ch2 */
#define GPIO1_B01_Ch2 DioConf_DioChannel_GPIO1_B01_Ch2

/** \brief Symbolic name for GPIO channel #435 GPIO1_B01_Ch3 */

#define DioConf_DioChannel_GPIO1_B01_Ch3 ((Dio_ChannelType) 435U)
/** \brief Shorter Symbolic name for GPIO channel #435 GPIO1_B01_Ch3 */
#define GPIO1_B01_Ch3 DioConf_DioChannel_GPIO1_B01_Ch3

/** \brief Symbolic name for GPIO channel #436 GPIO1_B01_Ch4 */

#define DioConf_DioChannel_GPIO1_B01_Ch4 ((Dio_ChannelType) 436U)
/** \brief Shorter Symbolic name for GPIO channel #436 GPIO1_B01_Ch4 */
#define GPIO1_B01_Ch4 DioConf_DioChannel_GPIO1_B01_Ch4

/** \brief Symbolic name for GPIO channel #437 GPIO1_B01_Ch5 */

#define DioConf_DioChannel_GPIO1_B01_Ch5 ((Dio_ChannelType) 437U)
/** \brief Shorter Symbolic name for GPIO channel #437 GPIO1_B01_Ch5 */
#define GPIO1_B01_Ch5 DioConf_DioChannel_GPIO1_B01_Ch5

/** \brief Symbolic name for GPIO channel #438 GPIO1_B01_Ch6 */

#define DioConf_DioChannel_GPIO1_B01_Ch6 ((Dio_ChannelType) 438U)
/** \brief Shorter Symbolic name for GPIO channel #438 GPIO1_B01_Ch6 */
#define GPIO1_B01_Ch6 DioConf_DioChannel_GPIO1_B01_Ch6

/** \brief Symbolic name for GPIO channel #439 GPIO1_B01_Ch7 */

#define DioConf_DioChannel_GPIO1_B01_Ch7 ((Dio_ChannelType) 439U)
/** \brief Shorter Symbolic name for GPIO channel #439 GPIO1_B01_Ch7 */
#define GPIO1_B01_Ch7 DioConf_DioChannel_GPIO1_B01_Ch7

/** \brief Symbolic name for GPIO channel #440 GPIO1_B01_Ch8 */

#define DioConf_DioChannel_GPIO1_B01_Ch8 ((Dio_ChannelType) 440U)
/** \brief Shorter Symbolic name for GPIO channel #440 GPIO1_B01_Ch8 */
#define GPIO1_B01_Ch8 DioConf_DioChannel_GPIO1_B01_Ch8

/** \brief Symbolic name for GPIO channel #441 GPIO1_B01_Ch9 */

#define DioConf_DioChannel_GPIO1_B01_Ch9 ((Dio_ChannelType) 441U)
/** \brief Shorter Symbolic name for GPIO channel #441 GPIO1_B01_Ch9 */
#define GPIO1_B01_Ch9 DioConf_DioChannel_GPIO1_B01_Ch9

/** \brief Symbolic name for GPIO channel #442 GPIO1_B01_Ch10 */

#define DioConf_DioChannel_GPIO1_B01_Ch10 ((Dio_ChannelType) 442U)
/** \brief Shorter Symbolic name for GPIO channel #442 GPIO1_B01_Ch10 */
#define GPIO1_B01_Ch10 DioConf_DioChannel_GPIO1_B01_Ch10

/** \brief Symbolic name for GPIO channel #443 GPIO1_B01_Ch11 */

#define DioConf_DioChannel_GPIO1_B01_Ch11 ((Dio_ChannelType) 443U)
/** \brief Shorter Symbolic name for GPIO channel #443 GPIO1_B01_Ch11 */
#define GPIO1_B01_Ch11 DioConf_DioChannel_GPIO1_B01_Ch11

/** \brief Symbolic name for GPIO channel #444 GPIO1_B01_Ch12 */

#define DioConf_DioChannel_GPIO1_B01_Ch12 ((Dio_ChannelType) 444U)
/** \brief Shorter Symbolic name for GPIO channel #444 GPIO1_B01_Ch12 */
#define GPIO1_B01_Ch12 DioConf_DioChannel_GPIO1_B01_Ch12

/** \brief Symbolic name for GPIO channel #445 GPIO1_B01_Ch13 */

#define DioConf_DioChannel_GPIO1_B01_Ch13 ((Dio_ChannelType) 445U)
/** \brief Shorter Symbolic name for GPIO channel #445 GPIO1_B01_Ch13 */
#define GPIO1_B01_Ch13 DioConf_DioChannel_GPIO1_B01_Ch13

/** \brief Symbolic name for GPIO channel #446 GPIO1_B01_Ch14 */

#define DioConf_DioChannel_GPIO1_B01_Ch14 ((Dio_ChannelType) 446U)
/** \brief Shorter Symbolic name for GPIO channel #446 GPIO1_B01_Ch14 */
#define GPIO1_B01_Ch14 DioConf_DioChannel_GPIO1_B01_Ch14

/** \brief Symbolic name for GPIO channel #447 GPIO1_B01_Ch15 */

#define DioConf_DioChannel_GPIO1_B01_Ch15 ((Dio_ChannelType) 447U)
/** \brief Shorter Symbolic name for GPIO channel #447 GPIO1_B01_Ch15 */
#define GPIO1_B01_Ch15 DioConf_DioChannel_GPIO1_B01_Ch15

/** \brief Symbolic name for GPIO channel #448 GPIO1_B01_Ch16 */

#define DioConf_DioChannel_GPIO1_B01_Ch16 ((Dio_ChannelType) 448U)
/** \brief Shorter Symbolic name for GPIO channel #448 GPIO1_B01_Ch16 */
#define GPIO1_B01_Ch16 DioConf_DioChannel_GPIO1_B01_Ch16

/** \brief Symbolic name for GPIO channel #449 GPIO1_B01_Ch17 */

#define DioConf_DioChannel_GPIO1_B01_Ch17 ((Dio_ChannelType) 449U)
/** \brief Shorter Symbolic name for GPIO channel #449 GPIO1_B01_Ch17 */
#define GPIO1_B01_Ch17 DioConf_DioChannel_GPIO1_B01_Ch17

/** \brief Symbolic name for GPIO channel #450 GPIO1_B01_Ch18 */

#define DioConf_DioChannel_GPIO1_B01_Ch18 ((Dio_ChannelType) 450U)
/** \brief Shorter Symbolic name for GPIO channel #450 GPIO1_B01_Ch18 */
#define GPIO1_B01_Ch18 DioConf_DioChannel_GPIO1_B01_Ch18

/** \brief Symbolic name for GPIO channel #451 GPIO1_B01_Ch19 */

#define DioConf_DioChannel_GPIO1_B01_Ch19 ((Dio_ChannelType) 451U)
/** \brief Shorter Symbolic name for GPIO channel #451 GPIO1_B01_Ch19 */
#define GPIO1_B01_Ch19 DioConf_DioChannel_GPIO1_B01_Ch19

/** \brief Symbolic name for GPIO channel #452 GPIO1_B01_Ch20 */

#define DioConf_DioChannel_GPIO1_B01_Ch20 ((Dio_ChannelType) 452U)
/** \brief Shorter Symbolic name for GPIO channel #452 GPIO1_B01_Ch20 */
#define GPIO1_B01_Ch20 DioConf_DioChannel_GPIO1_B01_Ch20

/** \brief Symbolic name for GPIO channel #453 GPIO1_B01_Ch21 */

#define DioConf_DioChannel_GPIO1_B01_Ch21 ((Dio_ChannelType) 453U)
/** \brief Shorter Symbolic name for GPIO channel #453 GPIO1_B01_Ch21 */
#define GPIO1_B01_Ch21 DioConf_DioChannel_GPIO1_B01_Ch21

/** \brief Symbolic name for GPIO channel #454 GPIO1_B01_Ch22 */

#define DioConf_DioChannel_GPIO1_B01_Ch22 ((Dio_ChannelType) 454U)
/** \brief Shorter Symbolic name for GPIO channel #454 GPIO1_B01_Ch22 */
#define GPIO1_B01_Ch22 DioConf_DioChannel_GPIO1_B01_Ch22

/** \brief Symbolic name for GPIO channel #455 GPIO1_B01_Ch23 */

#define DioConf_DioChannel_GPIO1_B01_Ch23 ((Dio_ChannelType) 455U)
/** \brief Shorter Symbolic name for GPIO channel #455 GPIO1_B01_Ch23 */
#define GPIO1_B01_Ch23 DioConf_DioChannel_GPIO1_B01_Ch23

/** \brief Symbolic name for GPIO channel #456 GPIO1_B01_Ch24 */

#define DioConf_DioChannel_GPIO1_B01_Ch24 ((Dio_ChannelType) 456U)
/** \brief Shorter Symbolic name for GPIO channel #456 GPIO1_B01_Ch24 */
#define GPIO1_B01_Ch24 DioConf_DioChannel_GPIO1_B01_Ch24

/** \brief Symbolic name for GPIO channel #457 GPIO1_B01_Ch25 */

#define DioConf_DioChannel_GPIO1_B01_Ch25 ((Dio_ChannelType) 457U)
/** \brief Shorter Symbolic name for GPIO channel #457 GPIO1_B01_Ch25 */
#define GPIO1_B01_Ch25 DioConf_DioChannel_GPIO1_B01_Ch25

/** \brief Symbolic name for GPIO channel #458 GPIO1_B01_Ch26 */

#define DioConf_DioChannel_GPIO1_B01_Ch26 ((Dio_ChannelType) 458U)
/** \brief Shorter Symbolic name for GPIO channel #458 GPIO1_B01_Ch26 */
#define GPIO1_B01_Ch26 DioConf_DioChannel_GPIO1_B01_Ch26

/** \brief Symbolic name for GPIO channel #459 GPIO1_B01_Ch27 */

#define DioConf_DioChannel_GPIO1_B01_Ch27 ((Dio_ChannelType) 459U)
/** \brief Shorter Symbolic name for GPIO channel #459 GPIO1_B01_Ch27 */
#define GPIO1_B01_Ch27 DioConf_DioChannel_GPIO1_B01_Ch27

/** \brief Symbolic name for GPIO channel #460 GPIO1_B01_Ch28 */

#define DioConf_DioChannel_GPIO1_B01_Ch28 ((Dio_ChannelType) 460U)
/** \brief Shorter Symbolic name for GPIO channel #460 GPIO1_B01_Ch28 */
#define GPIO1_B01_Ch28 DioConf_DioChannel_GPIO1_B01_Ch28

/** \brief Symbolic name for GPIO channel #461 GPIO1_B01_Ch29 */

#define DioConf_DioChannel_GPIO1_B01_Ch29 ((Dio_ChannelType) 461U)
/** \brief Shorter Symbolic name for GPIO channel #461 GPIO1_B01_Ch29 */
#define GPIO1_B01_Ch29 DioConf_DioChannel_GPIO1_B01_Ch29

/** \brief Symbolic name for GPIO channel #462 GPIO1_B01_Ch30 */

#define DioConf_DioChannel_GPIO1_B01_Ch30 ((Dio_ChannelType) 462U)
/** \brief Shorter Symbolic name for GPIO channel #462 GPIO1_B01_Ch30 */
#define GPIO1_B01_Ch30 DioConf_DioChannel_GPIO1_B01_Ch30

/** \brief Symbolic name for GPIO channel #463 GPIO1_B01_Ch31 */

#define DioConf_DioChannel_GPIO1_B01_Ch31 ((Dio_ChannelType) 463U)
/** \brief Shorter Symbolic name for GPIO channel #463 GPIO1_B01_Ch31 */
#define GPIO1_B01_Ch31 DioConf_DioChannel_GPIO1_B01_Ch31

/** \brief Symbolic name for GPIO channel #464 GPIO1_B23_Ch0 */

#define DioConf_DioChannel_GPIO1_B23_Ch0 ((Dio_ChannelType) 464U)
/** \brief Shorter Symbolic name for GPIO channel #464 GPIO1_B23_Ch0 */
#define GPIO1_B23_Ch0 DioConf_DioChannel_GPIO1_B23_Ch0

/** \brief Symbolic name for GPIO channel #465 GPIO1_B23_Ch1 */

#define DioConf_DioChannel_GPIO1_B23_Ch1 ((Dio_ChannelType) 465U)
/** \brief Shorter Symbolic name for GPIO channel #465 GPIO1_B23_Ch1 */
#define GPIO1_B23_Ch1 DioConf_DioChannel_GPIO1_B23_Ch1

/** \brief Symbolic name for GPIO channel #466 GPIO1_B23_Ch2 */

#define DioConf_DioChannel_GPIO1_B23_Ch2 ((Dio_ChannelType) 466U)
/** \brief Shorter Symbolic name for GPIO channel #466 GPIO1_B23_Ch2 */
#define GPIO1_B23_Ch2 DioConf_DioChannel_GPIO1_B23_Ch2

/** \brief Symbolic name for GPIO channel #467 GPIO1_B23_Ch3 */

#define DioConf_DioChannel_GPIO1_B23_Ch3 ((Dio_ChannelType) 467U)
/** \brief Shorter Symbolic name for GPIO channel #467 GPIO1_B23_Ch3 */
#define GPIO1_B23_Ch3 DioConf_DioChannel_GPIO1_B23_Ch3
/* @} */


/* Requirements : MCAL-1101, MCAL-1102, MCAL-1136*/
/**
 *  \name Symbolic name of DIO Channel Groups
 *
 *  This is the symbolic name of DIO channel groups generated from configurator
 *  The definition of the channel groups is present in the generated
 *  Dio_Lcfg.c file. The other modules will invoke the Dio_channelGroup APIs
 *  using the symbolic name defines
 *
 *  @{
 */
#define DioConf_DioChannelGroup_WKUP_GPIO0_B01_Bit0_1_PTR ((const Dio_ChannelGroupType *)(DioConfig_WKUP_GPIO0_B01_ChannelGroupRef[0]))
#define WKUP_GPIO0_B01_Bit0_1_PTR DioConf_DioChannelGroup_WKUP_GPIO0_B01_Bit0_1_PTR
/* @} */

/**
 *  \name DIO DEM Error codes to report
 *
 *  Pre-compile switches for enabling/disabling DEM events
 *  @{
 */
#define DemConf_DemEventParameter_DIO_DEM_NO_EVENT (0xFFFFU)
#define DIO_DEM_NO_EVENT DemConf_DemEventParameter_DIO_DEM_NO_EVENT

/* DIO IO check Dem IDs */
#ifndef DIO_WRITE_CHANNEL_EVENT_ID
    #define DIO_WRITE_CHANNEL_EVENT_ID  (DemConf_DemEventParameter_DIO_WRITE_CHANNEL_EVENT_ID)
#endif
#ifndef DIO_WRITE_PORT_EVENT_ID
    #define DIO_WRITE_PORT_EVENT_ID     (DemConf_DemEventParameter_DIO_WRITE_PORT_EVENT_ID)
#endif

/* @} */


/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */
/* None */

/* ========================================================================== */
/*                          Global Declarations                               */
/* ========================================================================== */

/** \brief Reference to channel group structure for WKUP_GPIO0_B01
  *  created at config time */
extern P2CONST(void, AUTOMATIC, DIO_CONST) DioConfig_WKUP_GPIO0_B01_ChannelGroupRef[1];

/** \brief Reference to channel group structure for WKUP_GPIO0_B23
  *  created at config time */
extern P2CONST(void, AUTOMATIC, DIO_CONST) DioConfig_WKUP_GPIO0_B23_ChannelGroupRef[0];

/** \brief Reference to channel group structure for GPIO0_B01
  *  created at config time */
extern P2CONST(void, AUTOMATIC, DIO_CONST) DioConfig_GPIO0_B01_ChannelGroupRef[0];

/** \brief Reference to channel group structure for GPIO0_B23
  *  created at config time */
extern P2CONST(void, AUTOMATIC, DIO_CONST) DioConfig_GPIO0_B23_ChannelGroupRef[0];

/** \brief Reference to channel group structure for GPIO0_B45
  *  created at config time */
extern P2CONST(void, AUTOMATIC, DIO_CONST) DioConfig_GPIO0_B45_ChannelGroupRef[0];

/** \brief Reference to channel group structure for GPIO1_B01
  *  created at config time */
extern P2CONST(void, AUTOMATIC, DIO_CONST) DioConfig_GPIO1_B01_ChannelGroupRef[0];

/** \brief Reference to channel group structure for GPIO1_B23
  *  created at config time */
extern P2CONST(void, AUTOMATIC, DIO_CONST) DioConfig_GPIO1_B23_ChannelGroupRef[0];


#if (STD_ON == DIO_DEV_ERROR_DETECT)











    
    


/** \brief Channel mask for enabled pins in the available ports */
extern CONST (uint32,DIO_CONST) Dio_ConfigValidChannelMask[30U];

#endif /* (STD_ON == DIO_DEV_ERROR_DETECT) */

#ifdef __cplusplus
}
#endif

#endif  /* #ifndef DIO_CFG_H_ */

/* @} */

