/*
*
* Copyright (c) 2019 Texas Instruments Incorporated
*
* All rights reserved not granted herein.
*
* Limited License.
*
* Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive
* license under copyrights and patents it now or hereafter owns or controls to make,
* have made, use, import, offer to sell and sell ("Utilize") this software subject to the
* terms herein.  With respect to the foregoing patent license, such license is granted
* solely to the extent that any such patent is necessary to Utilize the software alone.
* The patent license shall not apply to any combinations which include this software,
* other than combinations with devices manufactured by or for TI ("TI Devices").
* No hardware patent is licensed hereunder.
*
* Redistributions must preserve existing copyright notices and reproduce this license
* (including the above copyright notice and the disclaimer and (if applicable) source
* code license limitations below) in the documentation and/or other materials provided
* with the distribution
*
* Redistribution and use in binary form, without modification, are permitted provided
* that the following conditions are met:
*
* *       No reverse engineering, decompilation, or disassembly of this software is
* permitted with respect to any software provided in binary form.
*
* *       any redistribution and use are licensed by TI for use only with TI Devices.
*
* *       Nothing shall obligate TI to provide you with source code for the software
* licensed and provided to you in object code.
*
* If software source code is provided to you, modification and redistribution of the
* source code are permitted provided that the following conditions are met:
*
* *       any redistribution and use of the source code, including any resulting derivative
* works, are licensed by TI for use only with TI Devices.
*
* *       any redistribution and use of any object code compiled from the source code
* and any resulting derivative works, are licensed by TI for use only with TI Devices.
*
* Neither the name of Texas Instruments Incorporated nor the names of its suppliers
*
* may be used to endorse or promote products derived from this software without
* specific prior written permission.
*
* DISCLAIMER.
*
* THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

/**
 *  \file     IntRtr_Cfg.h
 *
 *  \brief    Defines the router configurations used in example / demo
 *              applications.
 *
 */

#ifndef INTRTR_CFG_H_
#define INTRTR_CFG_H_

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */
#include "Std_Types.h"

#ifdef __cplusplus
extern "C"
{
#endif

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */

/** \brief Interrupt ID's for AM65x SoC
 *
 */
#define INTRTR_CFG_TOTAL_INPUT_INT_SUPPORTED    (192U)
/**< Total number of inputs interrupts supported */
#define INTRTR_CFG_TOTAL_OUTPUT_INT_SUPPORTED   (64U)
/**< Total number of output interrupts supported */

#define NAVSS0_INTR_ROUTER0_TOTAL_INPUT_INT_SUPPORTED   (440U)
/**< Total number of inputs interrupts supported */
#define NAVSS0_INTR_ROUTER0_TOTAL_OUTPUT_INT_SUPPORTED  (512U)
/**< Total number of inputs interrupts supported */

#define INTRTR_CFG_START_LEVEL_INT_NUMBER \
                                    (CSL_MCU0_INTR_MAIN2MCU_LVL_INTR0_OUTL_0)
/**< Total number of LEVEL output interrupts supported */
#define INTRTR_CFG_START_PULSE_INT_NUMBER \
                                    (CSL_MCU0_INTR_MAIN2MCU_PULSE_INTR0_OUTP_0)
/**< Total number of PULSE output interrupts supported */

#define INTRTR_CFG_MAIN_DOMAIN_MCPSI_1_OUT_INT_NO   (0U)
/**< Main domain McSPI instance 1 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_MCPSI_2_OUT_INT_NO   (1U)
/**< Main domain McSPI instance 2 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_MCPSI_3_OUT_INT_NO   (2U)
/**< Main domain McSPI instance 3 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_MCPSI_4_OUT_INT_NO   (3U)
/**< Main domain McSPI instance 4 interrupt is routed on output int no */

#define INTRTR_CFG_MAIN_DOMAIN_TIMER_1_OUT_INT_NO   (4U)
/**< Main domain Timer instance 1 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_TIMER_2_OUT_INT_NO   (5U)
/**< Main domain Timer instance 2 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_TIMER_3_OUT_INT_NO   (6U)
/**< Main domain Timer instance 3 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_TIMER_4_OUT_INT_NO   (7U)
/**< Main domain Timer instance 4 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_TIMER_5_OUT_INT_NO   (8U)
/**< Main domain Timer instance 5 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_TIMER_6_OUT_INT_NO   (9U)
/**< Main domain Timer instance 6 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_TIMER_7_OUT_INT_NO   (10U)
/**< Main domain Timer instance 7 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_TIMER_8_OUT_INT_NO   (11U)
/**< Main domain Timer instance 8 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_TIMER_9_OUT_INT_NO   (12U)
/**< Main domain Timer instance 9 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_TIMER_10_OUT_INT_NO  (13U)
/**< Main domain Timer instance 10 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_TIMER_11_OUT_INT_NO  (14U)
/**< Main domain Timer instance 11 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_TIMER_12_OUT_INT_NO  (15U)
/**< Main domain Timer instance 12 interrupt is routed on output int no */
#define INTRTR_CFG_MAIN_DOMAIN_MBX_CLST0_USR1_OUT_INT_NO  (16U)
/**< Main domain MAILBOX 0, Cluster 0 and User 1 */
#define INTRTR_CFG_MAIN_DOMAIN_MBX_CLST2_USR0_OUT_INT_NO  (17U)
/**< Main domain MAILBOX 0, Cluster 2 and User 0 */

/* IPC / Mailbox specific configurations */
/**< MCU 1 0, used cluster 0, user 1 to receive messages from MCU 1 1 */
#define NAVSS0_INTR_ROUTER0_INPUT_CLUSTER_0_USER_1_MCU_1_1  (437U)
/**< Router input number */
#define NAVSS0_INTR_ROUTER0_OUTPUT_INT_NUM_MUC_1_1          (121U)
/**< Router output number */
#define MAIN2MCU_INTRTR_INPUT_NUM_MUC_1_1                   (185U)
/**< Router input number */

/**< MPU 1 0, used cluster 0, user 0 to receive messages from MPU 1 0 */
#define NAVSS0_INTR_ROUTER0_INPUT_CLUSTER_0_USER_0_MPU_1_0  (437U)
/**< Router input number */
#define NAVSS0_INTR_ROUTER0_OUTPUT_INT_NUM_MPU_1_0          (121U)
/**< Router output number */
#define MAIN2MCU_INTRTR_INPUT_NUM_MPU_1_0                   (185U)
/**< Router input number */

#ifdef __cplusplus
}
#endif

#endif  /* #ifndef INTRTR_CFG_H_ */
