/*
*
* Copyright (c) 2019 Texas Instruments Incorporated
*
* All rights reserved not granted herein.
*
* Limited License.
*
* Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive
* license under copyrights and patents it now or hereafter owns or controls to make,
* have made, use, import, offer to sell and sell ("Utilize") this software subject to the
* terms herein.  With respect to the foregoing patent license, such license is granted
* solely to the extent that any such patent is necessary to Utilize the software alone.
* The patent license shall not apply to any combinations which include this software,
* other than combinations with devices manufactured by or for TI ("TI Devices").
* No hardware patent is licensed hereunder.
*
* Redistributions must preserve existing copyright notices and reproduce this license
* (including the above copyright notice and the disclaimer and (if applicable) source
* code license limitations below) in the documentation and/or other materials provided
* with the distribution
*
* Redistribution and use in binary form, without modification, are permitted provided
* that the following conditions are met:
*
* *       No reverse engineering, decompilation, or disassembly of this software is
* permitted with respect to any software provided in binary form.
*
* *       any redistribution and use are licensed by TI for use only with TI Devices.
*
* *       Nothing shall obligate TI to provide you with source code for the software
* licensed and provided to you in object code.
*
* If software source code is provided to you, modification and redistribution of the
* source code are permitted provided that the following conditions are met:
*
* *       any redistribution and use of the source code, including any resulting derivative
* works, are licensed by TI for use only with TI Devices.
*
* *       any redistribution and use of any object code compiled from the source code
* and any resulting derivative works, are licensed by TI for use only with TI Devices.
*
* Neither the name of Texas Instruments Incorporated nor the names of its suppliers
*
* may be used to endorse or promote products derived from this software without
* specific prior written permission.
*
* DISCLAIMER.
*
* THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

/**
 *  \file     Wdg_Cfg.h
 *
 *  \brief    This file contains generated pre compile configuration file
 *            for WDG MCAL driver
 *
 */

/**
 *  \defgroup MCAL_WDG_CFG WDG Configuration
 *
 *  This files defines WDG MCAL configuration structures
 *  @{
 */

/******************************************************************************
    Project: Wdg
    Date   : 2019-06-29 13:22:49
    This file is generated by EB Tresos
    Do not modify this file, otherwise the software may behave in unexpected way.
*******************************************************************************/

/* Protection against multiple inclusion */
#ifndef WDG_CFG_H
#define WDG_CFG_H

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */
#include "Dem.h"

#ifdef __cplusplus
extern "C" {
#endif

/* ========================================================================== */
/*                    Exported Preprocessor #define Constants                 */
/* ========================================================================== */






/* Design : DES_WDG_005 */
/**
*  \brief WDG Build Variant.
*   Build variants.(i.e Pre Compile,Link Time And Post Build)
*/
#define WDG_VARIANT_PRE_COMPILE (STD_ON)



/** \brief Pre Compile config macro name. */
#define WDG_INIT_CONFIG_PC       WdgSettingsConfig



/** \brief Enable/Disable Development Error Detection */
#define WDG_DEV_ERROR_DETECT       (STD_ON)

/** \brief Enable/Disable Version Info API */
#define WDG_GET_VERSION_INFO_API   (STD_ON)

/** \brief Enable/Disable Register Readback API */
#define WDG_REGISTER_READBACK_API  (STD_ON)

/** \brief Watchdog Maximum Timeout
  *        x 1000U - converting from secs so milli secs */
#define WDG_MAX_TIMEOUT            ((uint16)65535)


/** \brief RTI Clock Frequency */
#define WDG_RTI_FREQUENCY          ((uint32)12500000)

/** \brief Watchdog Disable Allowed*/
#define WDG_DISABLE_ALLOWED        (STD_OFF)

/**
 *  \name WDG Module Instance Id
 *
 *  Symbolic names for module instance id
 *  @{
 */
/** \brief Wdg Module Instance Id, always 0 as only one instance in this case */
#define WdgConf_WdgGeneral_WdgIndex_0 (0U)

/* @} */

/* Requirements : MCAL-2577 */
/**
 *  \name WDG DEM Error codes to report
 *
 *  Pre-compile switches for enabling/disabling DEM events
 *  @{
 */
#define DemConf_DemEventParameter_WDG_DEM_NO_EVENT (0xFFFFU)
#define WDG_DEM_NO_EVENT DemConf_DemEventParameter_WDG_DEM_NO_EVENT

#ifndef WDG_E_MODE_FAILED
/** \brief Mode failed */
#define WDG_E_MODE_FAILED          DemConf_DemEventParameter_WDG_E_MODE_FAILED
#endif
/* Requirements : MCAL-2580 */
#ifndef WDG_E_DISABLE_REJECTED
/** \brief Disabled rejected */
#define WDG_E_DISABLE_REJECTED     DemConf_DemEventParameter_WDG_E_DISABLE_REJECTED
#endif

/* @} */

/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */

/** \brief WDG Configuration struct declaration */
extern const struct Wdg_ConfigType_s WdgSettingsConfig;

/** \brief WDG Configuration pre compile struct declaration */
extern const struct Wdg_ConfigType_PC_s WdgSettingsConfig_PC;


#ifdef __cplusplus
}
#endif

#endif  /* WDG_CFG_H */

/* @} */

/*******************************************************************************
 *  End of File: Wdg_Cfg.h
 ******************************************************************************/
